{
  Entered into the public domain.  Use at your own risk.

  Submitted by:  Mike Armstrong
                 1637 Winding Way
                 Richmond, VA 23235
                 72740,1145

  All comments and suggestions gratefully appreciated (I'm always
  trying to learn more).

  For more information, look at the TDCB entry in the Borland Pascal
  help file.  It provides a good explanation of what each routine does.

  Good luck and I hope it's useful
}

Unit Comm;

{ A Comm object for Windows.  To use, declare a TCommPort variable
  initialized with a Port Number, RX Buffer size,and TX Buffer size.
  Once initialized, set the Baud, Parity, Data bits, and Stop bits. }

Interface

Uses Objects, WinTypes, WinProcs, Strings;

{   Communications Routines for Windows }

{ Rtruns a desriptive string if there is a failure opening the comm port }
function OpenCommError (intError : Integer): String;

Const
  None = 0;
  Odd = 1;
  Even = 2;
  Mark = 3;

  Space = 4;
  Stop1 = 0;
  Stop1_5 = 1;
  Stop2 = 2;  { Stop Bits 0,1,2 = 1, 1.5, 2 }

Type
  PCommPort = ^TCommPort;
  TCommPort = object(TObject)
    intComID  : Integer;  { ID of open Comm Port }
    Port: Byte;
    Baud: Word;
    Parity: Byte;
    DataBits: Byte;
    StopBits: Byte;
    XOnChar,
    XOffChar  : Char;
    XonLim: Word;          { Point at which X-On is sent (threshold value)  }
    XoffLim: Word;         { Point at which X-Off is sent (threshold value) }
    PeChar: char;          { Parity error replacement character }
    EofChar: char;         { End-of-Input character }

    LastError : Word;      { Tracks the last error that occured }
    RXBuffer,
    TXBuffer: Word;        { Rx and TX Buffer Sizes }
    DCB       : TDCB;      { DCB associated with this CommPort object }
    ComStat   : TComStat;

    constructor Init(APort: Byte; ARXBuffer: Word; ATXBuffer: Word);
    destructor Done; virtual;
    function ReadCommChar: Char;
    function ReadCommString: String;
    function ReadCommFile(strFileName: String): LongInt;
    function WriteCommChar(chrOut: Char): Integer;
    function WriteCommString(strOut:String):Integer;
    function WriteCommFile(strFileName: String): Integer;
    function XOff: Integer;
    function XOn: Integer;
    function RX_FlowControl(OnOff: Boolean): Integer;
    function TX_FlowControl(OnOff: Boolean): Integer;
    function CommError: Integer;
    function RXCount: Word;
    function TXCount: Word;
    function RXFlush: Integer;
    function TXFlush: Integer;
    function SetXOnChar(AXOnChar: Char): Integer;
    function SetXOffChar(AXOffChar: Char): Integer;
    function SetXOnLim(AXOnLim: Word): Integer;
    function SetXOffLim(AXOffLim: Word): Integer;
    function ParityErrReplacement(OnOff: Boolean): Integer;
    function SetParityErrChar(APEChar: Char): Integer;
    function SetEOFChar(AEOFChar: Char): Integer;
    function SetDTR(OnOff: Boolean): Integer;
    function SetRTS(OnOff: Boolean): Integer;
    function GetState: Integer;
    function SetBaudRate(ABaud : Word): Integer;
    function SetParity(AParity: Byte): Integer;
    function SetDataBits(ADataBits: Byte): Integer;
    function SetStopBits(AStopBits: Byte): Integer;
  end;

var
  strCommError : String;   { global variable }

{******************************************************************}

                               Implementation

{******************************************************************}

Const
  WaitTime = 90000; { Used to set loop value for timeout function
                      when reading comm port }

{ Returns comm error and clears comm error.  Comm will shut down if an
  error occurs.  Comm will stay down until the error is cleared by
  calling GetCommError.  Call this after each Comm function call
  to be on the safe side.  Sets the global variable strCommError to
  a descriptive string of the error which occured }
function TCommPort.CommError: Integer;
Var
  intResult : Integer;
begin
    strCommError := 'No Error';
    intResult := GetCommError(intComID, ComStat);

    If (intResult And CE_Break) <> 0 Then
        strCommError := 'Hardware Break';
    If (intResult And CE_CTSTO) <> 0 Then
        strCommError := 'CTS Time Out';
    If (intResult And CE_DNS) <> 0 Then
        strCommError := 'Parallel Device NOT Selected';
    If (intResult And CE_DSRTO) <> 0 Then
        strCommError := 'DSR Time Out';
    If (intResult And CE_Frame) <> 0 Then
        strCommError := 'Framing Error';
    If (intResult And CE_IOE) <> 0 Then
        strCommError := 'I/O Error (Parallel Device)';
    If (intResult And CE_Mode) <> 0 Then
        strCommError := 'Mode NOT Supported or Invalid CID';
    If (intResult And CE_OOP) <> 0 Then
        strCommError := 'Device Out of Paper';
    If (intResult And CE_OverRun) <> 0 Then
        strCommError := 'Character lost due to overun';
    If (intResult And CE_PTO) <> 0 Then
        strCommError := 'Parallel device timed out';
    If (intResult And CE_RLSDTO) <> 0 Then
        strCommError := 'RLSD Time Out';
    If (intResult And CE_RXOver) <> 0 Then
        strCommError := 'Receive Queue Overflow';
    If (intResult And CE_RXParity) <> 0 Then
        strCommError := 'Receive Parity Error';
    If (intResult And CE_TXFull) <> 0 Then
        strCommError := 'Transmit Buffer Full';
    LastError := intResult;
    CommError := intResult;
End;

destructor TCommPort.Done;
var
    intResult : Integer;
begin
  { Flush the buffers and close the com port...}
    intResult := TXFlush;
    If intResult <> 0 Then
        intResult := CommError
    Else
    Begin
      intResult := RXFlush;
      If intResult <> 0 Then
        intResult := CommError
      Else
      Begin
        intResult := CloseComm(intComID);
        If intResult <> 0 Then
           intResult := CommError;

      End;
    End;
    inherited Done;
end;

constructor TCommPort.Init(APort: Byte; ARXBuffer: Word; ATXBuffer: Word);
var
  intResult : Integer;
  strTemp,
  strDefaults : String;
  pComDev:  Array[0..32] of Char;
begin
   Inherited Init;
   RXBuffer := ARXBuffer;
   TXBuffer := ATXBuffer;
   LastError := 0;
   Port := APort;
   XOnChar := Chr(17);
   XOffChar := Chr(19);
   Str(APort, strTemp);
   strDefaults := 'COM' + strTemp + ':';
   StrPCopy(pComDev, strDefaults);
   intComID := OpenComm(pComDev, RXBuffer, TXBuffer);  { port, in buffer size, out buffer size }
   If intComID >= 0 Then
   Begin
    { Flush the buffers }
     intResult := TXFlush;
     intResult := RXFlush;
     GetState;
   End
   LastError := intComID;
   { Always check LastError to see if there was an error (LastError < 0).
     If so, check OpenCommError for a description }
end;

{ Waits until a character is available, or until a WaitTime count has passed.
  If a character becomes available before the WaitTime counter has overflowed,
  the character is returned.  Otherwise, returns Chr(0) if no char available }
function TCommPort.ReadCommChar: Char;
var
  pIn : Array[0..255] of Char;
  intResult : Integer;
  lngTimeOut : LongInt;
  chrIn : Char;
begin
  lngTimeOut := 0;
  chrIn := Chr(0);
  LastError := CommError;
  { Loop until a char is available (or until timed out) }
  While (ComStat.cbInQue < 1) And (lngTimeOut < WaitTime) DO
  Begin
    LastError := CommError;
    Inc(lngTimeOut);
  End;
  If lngTimeOut >= WaitTime Then
     MessageBox(0, 'Comm Port Timed Out', 'Comm Error', MB_ICONSTOP)
  Else
  Begin
    intResult := Abs(ReadComm(intComID, pIn, 1));
    If intResult <> 0 Then
       chrIn := pIn[0];
  End;
  ReadCommChar := chrIn;
End;

{ Dumps the Comm port to a file until no more characters are available }
function TCommPort.ReadCommFile(strFileName: String): LongInt;
var
  chrIn : Char;
  intResult : Integer;
  intReady : Integer;
  lngBytesRead : LongInt;
  strOut : String;
  OutFile: Text;
begin
  lngBytesRead := 0;
  Assign(OutFile, strFileName);
  {$I-}
  ReWrite(OutFile);
  {$I+}
  If IOResult <> 0 Then
     MessageBox(0, 'Unable to open file', 'File Error', MB_ICONSTOP)
  Else
  Begin
    strOut := '';
    chrIn := ReadCommChar;
    While chrIn <> Chr(0) Do
    Begin
      Inc(lngBytesRead );
      If Ord(chrIn) < 27 Then
      Begin
        WriteLn(OutFile, strOut);
        strOut := '';
      End
      Else
        strOut := strOut + chrIn;
    End;
  End;
  Close(OutFile);
  ReadCommFile := lngBytesRead;
end;

{ Reads a strings from the comm port.  Assumes any Control Character as
  end of line.  Be careful to use only when you know its a text file, as
  all characters < #27 are considered end-of-line markers }

function TCommPort.ReadCommString: String;
var
  chrIn : Char;
  strOut : String;
  Found : Boolean;
begin
  strOut := '';
  Found := False;
  chrIn := ReadCommChar;
  While (chrIn <> Chr(0)) And Not Found Do
  Begin
    If Ord(chrIn) < 27 Then
       Found := True
    Else
       strOut := strOut + chrIn;
  End;
  ReadCommString := strOut;
End;

function TCommPort.WriteCommChar(chrOut: Char): Integer;
var
  pOut: Array[0..1] Of Char;
begin
  pOut[0] := chrOut;
  WriteCommChar := WriteComm(intComID, pOut, 1);
end;

{ Outputs a string }
function TCommPort.WriteCommString(strOut:String):Integer;
var
  pOut : PChar;
begin
  GetMem(pOut, Length(strOut) + 1);
  StrPCopy(pOut, strOut);
  WriteCommString := WriteComm(intComID, pOut, Length(strOut));
  FreeMem(pOut, Length(strOut) + 1);
end;

{ Outputs a file to the comm port }
function TCommPort.WriteCommFile(strFileName: String): Integer;
var
  InFile: Text;
  strOut: String;
  intResult,
  intGood: Integer;
begin
  intGood := 0;
  Assign(InFile, strFileName);
  {$I-}
  Reset(InFile);
  {$I+}
  If IOResult <> 0 Then
  Begin
     MessageBox(0, 'Unable to write file', 'CommError', MB_ICONSTOP);
     intGood := -1;
  End
  Else
  While Not EOF(InFile) Do
  Begin
    ReadLn(InFile, strOut);
    intResult := WriteCommString(strOut);
    If intResult <> 0 Then
       intGood := intResult;
  End;
  WriteCommFile := intGood;
end;

{ Returns a count of bytes RX buffer }
function TCommPort.RXCount: Word;
begin
  CommError;
  RXCount := ComStat.cbInQue
end;

{ Returns a count of bytes TX buffer }
function TCommPort.TXCount: Word;
begin
  CommError;
  TXCount := ComStat.cbOutQue
end;

{ Flushes the TX buffer }
function TCommPort.TXFlush: Integer;
begin
  TXFlush := FlushComm(intComID, 0);
end;

{ Flushes the RX buffer }
function TCommPort.RXFlush: Integer;
begin
  RXFlush := FlushComm(intComID, 1);
end;

{ Sends the XOff Character }
function  TCommPort.XOff: Integer;
var
  intResult : Integer;
  intReturn : Integer;
begin
    intReturn := WriteCommChar(XOffChar);
    If intReturn <> 0 Then
        LastError := CommError;
    XOff := intReturn
end;

{ Sends the XOn Character }
function  TCommPort.XOn: Integer;
var
  intResult : Integer;
  intReturn : Integer;
begin
    intReturn := WriteCommChar(XOffChar);
    If intReturn <> 0 Then
        LastError := CommError;
    XOn := intReturn
end;

{ Turns RX Flow Control on or off }
function TCommPort.RX_FlowControl(OnOff: Boolean): Integer;
var
  intResult : Integer;
begin
  intResult := GetCommState(intComID, DCB);
  If intResult = 0 Then
  Begin
    If OnOff Then
       DCB.Flags := DCB.Flags OR $0100
    Else
       DCB.Flags := DCB.Flags AND $FEFF;
    intResult := SetCommState(DCB);
  End;
  RX_FlowControl := GetState;
end;

{ Turns TX Flow Control on or off }
function TCommPort.TX_FlowControl(OnOff: Boolean): Integer;
var
  intResult : Integer;
begin
  intResult := GetCommState(intComID, DCB);
  If intResult = 0 Then
  Begin
    If OnOff Then
       DCB.Flags := DCB.Flags OR $0200
    Else
       DCB.Flags := DCB.Flags AND $FDFF;
    intResult := SetCommState(DCB);
  End;
  TX_FlowControl := GetState;
end;

{ Set the character used for XOn }
function TCommPort.SetXOnChar(AXOnChar: Char): Integer;
var
  intResult : Integer;
begin
  intResult := GetCommState(intComID, DCB);
  If intResult = 0 Then
  Begin
    DCB.XOnChar := AXOnChar;
    intResult := SetCommState(DCB);
    XOnChar := AXOnChar;
  End;
  SetXOnChar := GetState;
end;

{ Set the character used for XOff }
function TCommPort.SetXOffChar(AXOffChar: Char): Integer;
var
  intResult : Integer;
begin
  intResult := GetCommState(intComID, DCB);
  If intResult = 0 Then
  Begin
    DCB.XOffChar := AXOffChar;
    intResult := SetCommState(DCB);
    XOffChar := AXOffChar;
  End;
  SetXOffChar := GetState;
end;

{ Set the limit before the comm port automatically sends a XOn }
function TCommPort.SetXOnLim(AXOnLim: Word): Integer;
var
  intResult : Integer;
begin
  intResult := GetCommState(intComID, DCB);
  If intResult = 0 Then
  Begin
    DCB.XOnLim := AXOnLim;
    intResult := SetCommState(DCB);
  End;
  SetXOnLim := GetState;
end;

{ Set the limit before the comm port automatically sends a XOff }
function TCommPort.SetXOffLim(AXOffLim: Word): Integer;
var
  intResult : Integer;
begin
  intResult := GetCommState(intComID, DCB);
  If intResult = 0 Then
  Begin
    DCB.XOffLim := AXOffLim;
    intResult := SetCommState(DCB);
  End;
  SetXOffLim := GetState;
end;

{ Replace parity error characters with a designated character }
function TCommPort.ParityErrReplacement(OnOff: Boolean): Integer;
var
  intResult : Integer;
begin
  intResult := GetCommState(intComID, DCB);
  If intResult = 0 Then
  Begin
    If OnOff Then
       DCB.Flags := DCB.Flags OR $0400
    Else
       DCB.Flags := DCB.Flags AND $FBFF;
    intResult := SetCommState(DCB);
  End;
  ParityErrReplacement := GetState;
end;

{ Replace parity error characters with this character if turned on by
  ParityErrReplacement(OnOff: Boolean) }
function TCommPort.SetParityErrChar(APEChar: Char): Integer;
var
  intResult : Integer;
begin
  intResult := GetCommState(intComID, DCB);
  If intResult = 0 Then
  Begin
    DCB.PEChar := APEChar;
    intResult := SetCommState(DCB);
  End;
  SetParityErrChar := GetState;
end;

{ Set character which siginifies end-of-file }
function TCommPort.SetEOFChar(AEOFChar: Char): Integer;
var
  intResult : Integer;
begin
  intResult := GetCommState(intComID, DCB);
  If intResult = 0 Then
  Begin
    DCB.EOFChar := AEOFChar;
    intResult := SetCommState(DCB);
  End;
  SetEOFChar := GetState;
end;


function TCommPort.SetDTR(OnOff: Boolean): Integer;
var
  intResult : Integer;
begin
  intResult := GetCommState(intComID, DCB);
  If intResult = 0 Then
  Begin
    If OnOff Then
       DCB.Flags := DCB.Flags OR $2000
    Else
       DCB.Flags := DCB.Flags AND $DFFF;
    intResult := SetCommState(DCB);
  End;
  SetDTR := GetState;
end;

function TCommPort.SetRTS(OnOff: Boolean): Integer;
var
  intResult : Integer;
begin
  intResult := GetCommState(intComID, DCB);
  If intResult = 0 Then
  Begin
    If OnOff Then
       DCB.Flags := DCB.Flags OR $4000
    Else
       DCB.Flags := DCB.Flags AND $BFFF;
    intResult := SetCommState(DCB);
  End;
  SetRTS := GetState;
end;

{ Get current state of Comm Port }
function TCommPort.GetState: Integer;
var
  intResult : Integer;
begin
  intResult := GetCommState(intComID, DCB);
  If intResult = 0 Then
  Begin
     Baud := DCB.BaudRate;
     DataBits := DCB.ByteSize;
     Parity := DCB.Parity;  { 0-4=None,Odd,Even,Mark,Space   }
     StopBits := DCB.StopBits;{ 0,1,2 = 1, 1.5, 2              }
     XOnChar := DCB.XOnChar;
     XOffChar := DCB.XOffChar;
     XOnLim := DCB.XOnLim;
     XOffLim := DCB.XOffLim;
     PEChar := DCB.PEChar;
     EOFChar := DCB.EOFChar;
  End;
  LastError := CommError;
  GetState := intResult;
end;

function TCommPort.SetBaudRate(ABaud : Word): Integer;
var
  intResult : Integer;
begin
  intResult := GetCommState(intComID, DCB);
  If intResult = 0 Then
  Begin
    DCB.BaudRate := ABaud;
    intResult := SetCommState(DCB);
  End;
  SetBaudRate := GetState;
end;

function TCommPort.SetParity(AParity: Byte): Integer;
var
  intResult : Integer;
begin
  intResult := GetCommState(intComID, DCB);
  If intResult = 0 Then
  Begin
    DCB.Parity := AParity;
    intResult := SetCommState(DCB);
  End;
  SetParity := GetState;
end;

function TCommPort.SetDataBits(ADataBits: Byte): Integer;
var
  intResult : Integer;
begin
  intResult := GetCommState(intComID, DCB);
  If intResult = 0 Then
  Begin
    DCB.ByteSize := ADataBits;
    intResult := SetCommState(DCB);
  End;
  SetDataBits := GetState;
end;

function TCommPort.SetStopBits(AStopBits: Byte): Integer;
var
  intResult : Integer;
begin
  intResult := GetCommState(intComID, DCB);
  If intResult = 0 Then
  Begin
    DCB.StopBits := AStopBits;
    intResult := SetCommState(DCB);
  End;
  SetStopBits := GetState;
end;

{ Returns a string specifying why an OpenComm failed }

function OpenCommError (intError : Integer): String;
var
  pError : Array[0..64] of Char;
begin
    Case intError of
          -1:  strCommError := 'The device identifier is invalid or unsupported';
         -12:  strCommError := 'The device does not support this Baud Rate';
         -11:  strCommError := 'The specified byte size is invalid';
          -5:  strCommError := 'The default parameters are in error';
         -10:  strCommError := 'The hardware is not available';
          -4:  strCommError := 'Cannot allocate memory for the queues';
          -3:  strCommError := 'The device is not open';
          -2:  strCommError := 'The device is already open';

         Else
              strCommError := 'Unknown error opening the device';
    End;
    StrPCopy(pError, strCommError);
    MessageBox(0, pError, 'Open Comm Device Error', MB_ICONHAND);
end;

Begin
End.
