
unit CTRL3D;

interface
uses
	WinTypes,WinProcs,OWindows,ODialogs;


type
	TCoordinateSet=
		record
			topLeftPt,topRightPt,botLeftPt,botRightPt:TPoint;
		end;

	PEdit3D=^TEdit3D;
	TEdit3D=
		object(TEdit)
			CoordSet				:TCoordinateSet;

			procedure SetupWindow; virtual;
			procedure wmPaint(var Msg:TMessage); virtual wm_First+wm_Paint;
		end;

	PComboBox3D=^TComboBox3D;
	TComboBox3D=
		object(TComboBox)
			CoordSet				:TCoordinateSet;

			procedure SetupWindow; virtual;
			procedure wmPaint(var Msg:TMessage); virtual wm_First+wm_Paint;
		end;

	PListBox3D=^TListBox3D;
	TListBox3D=
		object(TListBox)
			CoordSet				:TCoordinateSet;

			procedure SetupWindow; virtual;
			procedure wmPaint(var Msg:TMessage); virtual wm_First+wm_Paint;
		end;


implementation
procedure GetCoordinates(AParent:PWindowsObject;
								AControl:PControl;
								var ACoordSet:TCoordinateSet);
var
	topLeft,topRight,botLeft,botRight:longint;

begin
	{get control coordinates}
	with AControl^.Attr, ACoordSet do
	begin
		topLeft:=MakeLong(X,Y);
		topLeftPt:=MakePoint(topLeft);
		topRight:=MakeLong(X+W,Y);
		topRightPt:=MakePoint(topRight);
		botLeft:=MakeLong(X,Y+H);
		botLeftPt:=MakePoint(botLeft);
		botRight:=MakeLong(X+W,Y+H);
		botRightPt:=MakePoint(botRight);
	end;

	{convert screen coordinates to window coordinates}
	with ACoordSet do
	begin
		ScreenToClient(AParent^.hWindow,topLeftPt);
		ScreenToClient(AParent^.hWindow,topRightPt);
		ScreenToClient(AParent^.hWindow,botLeftPt);
		ScreenToClient(AParent^.hWindow,botRightPt);
	end;
end;

procedure Paint3DEffect(ACoordSet:TCoordinateSet;ADC:HDC);
var
	ThePen,OldPen:HPen;

begin
	{get gray pen}
	ThePen:=CreatePen(ps_Solid,1,RGB(128,128,128));
	OldPen:=SelectObject(ADC,ThePen);

	with ACoordSet do
	begin
		{top side}
		MoveTo(ADC,topLeftPt.x-2,topLeftPt.y-2);
		LineTo(ADC,topRightPt.x+2,topRightPt.y-2);
		MoveTo(ADC,topLeftPt.x-1,topLeftPt.y-1);
		LineTo(ADC,topRightPt.x+1,topRightPt.y-1);

		{left side}
		MoveTo(ADC,topLeftPt.x-2,topLeftPt.y-2);
		LineTo(ADC,botLeftPt.x-2,botLeftPt.y+2);
		MoveTo(ADC,topLeftPt.x-1,topLeftPt.y-1);
		LineTo(ADC,botLeftPt.x-1,botLeftPt.y+1);
	end;

	{delete gray pen, get white pen}
	DeleteObject(ThePen);
	ThePen:=CreatePen(ps_Solid,1,RGB(255,255,255));
	SelectObject(ADC,ThePen);

	with ACoordSet do
	begin
		{right side}
		MoveTo(ADC,topRightPt.x+1,topRightPt.y-1);
		LineTo(ADC,botRightPt.x+1,botRightPt.y+2);
		MoveTo(ADC,topRightPt.x,topRightPt.y);
		LineTo(ADC,botRightPt.x,botRightPt.y+1);

		{bottom side}
		MoveTo(ADC,botLeftPt.x-1,botLeftPt.y+1);
		LineTo(ADC,botRightPt.x+1,botRightPt.y+1);
		MoveTo(ADC,botLeftPt.x,botLeftPt.y);
		LineTo(ADC,botRightPt.x,botRightPt.y);
	end;

	{delete white pen}
	DeleteObject(ThePen);
	SelectObject(ADC,OldPen);
end;

procedure TEdit3D.SetupWindow;
begin
	inherited SetupWindow;
	GetCoordinates(Parent,@Self,CoordSet);
end;

procedure TEdit3D.wmPaint(var Msg:TMessage);
var
	DC:HDC;

begin
	DefWndProc(Msg);
	DC:=GetDC(Parent^.hWindow);
	Paint3DEffect(CoordSet,DC);
	ReleaseDC(Parent^.hWindow,DC);
end;

procedure TComboBox3D.SetupWindow;
begin
	inherited SetupWindow;
	GetCoordinates(Parent,@Self,CoordSet);
end;

procedure TComboBox3D.wmPaint(var Msg:TMessage);
var
	DC:HDC;

begin
	DefWndProc(Msg);
	DC:=GetDC(Parent^.hWindow);
	Paint3DEffect(CoordSet,DC);
	ReleaseDC(Parent^.hWindow,DC);
end;

procedure TListBox3D.SetupWindow;
begin
	inherited SetupWindow;
	GetCoordinates(Parent,@Self,CoordSet);
end;

procedure TListBox3D.wmPaint(var Msg:TMessage);
var
	DC:HDC;

begin
	DefWndProc(Msg);
	DC:=GetDC(Parent^.hWindow);
	Paint3DEffect(CoordSet,DC);
	ReleaseDC(Parent^.hWindow,DC);
end;

end.