program ListboxDrag;

{$R DRAGLIST.RES}

{ If you want to test the Single Item dragging capabilities, remove the '$' from the
  following DEFINE statement and recompile. }
{$DEFINE MULTI}

uses
  WinTypes, WinProcs, ODialogs, OWindows, Strings,
  DList;

const
  id_LB1 = 101;
  id_LB2 = 102;

type
  DragApplication = object(TApplication)
    procedure InitMainWindow; virtual;
  end;

 PDragWindow = ^DragWindow;
 DragWindow = object(TDlgWindow)
   hDrop,
   hNoDrop : hCursor;
{$IFDEF MULTI}
   hMultiDrop: hCursor;
   LB1, LB2 : PMultiDragListBox;
{$ELSE}
   LB1, LB2 : PDragListBox;
{$ENDIF}

   constructor Init(AParent : PWindowsObject; AName : PChar);
   procedure SetupWindow; virtual;
   procedure ItemsDropped(var Msg : TMessage);
     virtual wm_First + dm_Drop;
 end;

procedure DragApplication.InitMainWindow;
begin
  MainWindow:=New(PDragWindow, Init(nil,'MAINDLG'));
end;

constructor DragWindow.Init(AParent : PWindowsObject; AName : PChar);
begin
  inherited Init(AParent, AName);
  hDrop := LoadCursor(hInstance, 'DROP');
  hNoDrop := LoadCursor(hInstance,'NODROP');
{$IFDEF MULTI}
  hMultiDrop := LoadCursor(hInstance, 'MULTIDROP');
  LB1:=New(PMultiDragListBox,Init(@Self,id_LB1,30,40,100,100,hDrop,hNoDrop,hMultiDrop));
  LB2:=New(PMultiDragListBox,Init(@Self,id_LB2,180,40,100,100,hDrop,hNoDrop,hMultiDrop));
  LB1^.Attr.Style := LB1^.Attr.Style or lbs_MultipleSel;
  LB2^.Attr.Style := LB2^.Attr.Style or lbs_MultipleSel;
{$ELSE}
  LB1:=New(PDragListBox,Init(@Self,id_LB1,30,40,100,100,hDrop,hNoDrop));
  LB2:=New(PDragListBox,Init(@Self,id_LB2,180,40,100,100,hDrop,hNoDrop));
{$ENDIF}
end;

procedure DragWindow.SetupWindow;
const
  MultiName : PChar = 'ListBox Multi-Drag Example';
begin
  inherited SetUpWindow;
  LB1^.AddString('Item #1');
  LB1^.AddString('Item #2');
  LB1^.AddString('Item #3');
  LB2^.AddString('Item A');
  LB2^.AddString('Item B');
  LB2^.AddString('Item C');
{$IFDEF MULTI}
  SendMessage(hWindow, wm_SetText, 0, LongInt(MultiName));
{$ENDIF}
end;

procedure DragWindow.ItemsDropped(var Msg : TMessage);
var
{$IFDEF MULTI}
  Counter: integer;
  DS: PMultiDropStruct;
{$ELSE}
  DS: PDropStruct;
{$ENDIF}
  Buf : Array[0..10] of char;
begin
{$IFDEF MULTI}
  DS := PMultiDropStruct(Msg.lParam);
  for Counter := DS^.NumItems downto 1 do begin
    SendMessage(DS^.hSource, lb_GetText, DS^.ItemArray^[Counter], LongInt(@Buf));
    SendMessage(DS^.hDest, lb_AddString, 0, LongInt(@Buf));
    SendMessage(DS^.hSource, lb_DeleteString, DS^.ItemArray^[Counter], 0);
  end;
{$ELSE}
  DS := PDropStruct(Msg.lParam);
  SendMessage(DS^.hSource, lb_GetText, DS^.ItemIndex, LongInt(@Buf));
  SendMessage(DS^.hSource, lb_DeleteString, DS^.ItemIndex, 0);
  SendMessage(DS^.hDest, lb_AddString, 0, LongInt(@Buf));
{$ENDIF}
end;

(**** MAIN ****)
var
  DragApp : DragApplication;
begin
  DragApp.Init('DragApp');
  DragApp.Run;
  DragApp.Done;
end.