unit MScroll;

{$R MSCROLL.RES}

{--------------------------------------------------------------------------}
{                            MicroScroll Control                           }
{--------------------------------------------------------------------------}

interface

uses WinTypes,WinProcs,OWindows,ODialogs;

const wm_MicroScroll=111; { User defined message }

const ms_NoBtn=0; { No button pressed }
      ms_UpBtn=1; { Upper button pressed }
      ms_DnBtn=2; { Lower btuuton pressed }

type TMicroScroll=
       object(TControl)
         AssID:Word; { ID of the associated edit control }
         CRect:TRect;
         BtnPushed:Byte;
         BtnRect:array[ms_UpBtn..ms_DnBtn] of TRect;
         MouseOnBtn:Boolean;
         Step:LongInt;
         TimeCnt:Byte;
         constructor InitResource(AParent:PWindowsObject;
                                  ID,AID:Word;Stp:Real);
         procedure SetupWindow; virtual;
         procedure Draw(PaintDC:HDC;Push:Byte;DrawBk:Boolean);
         procedure WMPaint(var Msg:TMessage);
           virtual wm_First+wm_Paint;
         procedure WMLButtonDblClk(var Msg:TMessage);
           virtual wm_First+wm_LButtonDblClk;
         procedure WMSetFocus(var Msg:TMessage);
           virtual wm_First+wm_SetFocus;
         procedure WMKillFocus(var Msg:TMessage);
           virtual wm_First+wm_KillFocus;
         procedure WMTimer(var Msg:TMessage);
           virtual wm_First+wm_Timer;
         procedure WMLButtonDown(var Msg:TMessage);
           virtual wm_First+wm_LButtonDown;
         procedure WMMouseMove(var Msg:TMessage);
           virtual wm_First+wm_MouseMove;
         procedure WMLButtonUp(var Msg:TMessage);
           virtual wm_First+wm_LButtonUp;
         procedure WMNCPaint(var Msg:TMessage);
           virtual wm_First+wm_NCPaint;
       end;

type PMicroScroll=^TMicroScroll;

implementation

procedure BMDraw(DC:HDC;BMName:PChar;X,Y,W,H:Integer);

{-- Draws a Bitmap named BMName }

var MemDC:HDC;
    HBit,OldBitMap:HBitMap;

begin
     HBit:=LoadBitMap(HInstance,BMName);
     MemDC:=CreateCompatibleDC(DC);
     OldBitMap:=SelectObject(MemDC,HBit);
     BitBlt(DC,X,Y,W,H,MemDC,0,0,srcCopy);
     SelectObject(MemDC,OldBitMap);
     DeleteDC(MemDC);
     DeleteObject(HBit);
end;


constructor TMicroScroll.InitResource(AParent:PWindowsObject;
                                      ID,AID:Word;Stp:Real);

{-- ID: MiscroScroll ID
    AID: ID of associated Edit control
    Stp: Value of increment/decrement }

begin
     inherited InitResource(AParent,ID);
     AssID:=AID;
     BtnPushed:=ms_NoBtn;
     MouseOnBtn:=False;
     TimeCnt:=0;
     Step:=Trunc(Stp*100);
     { -- Dirty trick: lParam is a LongInt but in this way we can
       pass to Edit Ctrl increments with 2 decimal places }
end;

procedure TMicroScroll.SetupWindow;

begin
     inherited SetupWindow;
     GetClientRect(HWindow,CRect);
     Dec(CRect.Bottom);Dec(CRect.Right);
     Inc(CRect.Left);
     BtnRect[ms_UpBtn]:=CRect;
     BtnRect[ms_UpBtn].Bottom:=CRect.Bottom div 2;
     BtnRect[ms_DnBtn]:=CRect;
     BtnRect[ms_DnBtn].Top:=CRect.Bottom div 2 + 1;
end;

procedure TMicroScroll.Draw(PaintDC:HDC;Push:Byte;DrawBk:Boolean);

var BMOfs,i:Byte;
    OldPen,LtPen,GrPen,DkPen:HPen;

begin
     LtPen:=GetStockObject(White_Pen);
     GrPen:=CreatePen(ps_Solid,1,RGB(192,192,192));
     DkPen:=CreatePen(ps_Solid,1,RGB(128,128,128));
     MoveTo(PaintDC,CRect.Left-1,CRect.Top);
     LineTo(PaintDC,CRect.Left-1,CRect.Bottom+1);
     OldPen:=SelectObject(PaintDC,LtPen);
     for i:=ms_UpBtn to ms_DnBtn do
        with BtnRect[i] do
          if Push=i then
            begin
              SelectObject(PaintDC,GrPen);
              MoveTo(PaintDC,Right,Top+1);
              LineTo(PaintDC,Right,Bottom);
              LineTo(PaintDC,Left,Bottom);
              SelectObject(PaintDC,DkPen);
              LineTo(PaintDC,Left,Top);
              LineTo(PaintDC,Right+1,Top);
            end
          else
            begin
              SelectObject(PaintDC,DkPen);
              MoveTo(PaintDC,Right,Top+1);
              LineTo(PaintDC,Right,Bottom);
              LineTo(PaintDC,Left,Bottom);
              SelectObject(PaintDC,LtPen);
              MoveTo(PaintDC,Left,Bottom-1);
              LineTo(PaintDC,Left,Top);
              LineTo(PaintDC,Right,Top);
            end;
     if Push=ms_UpBtn then BMOfs:=1 else BMOfs:=0;
     with BtnRect[ms_UpBtn] do
       BMDraw(PaintDC,'MSUP',Right div 2-2,Bottom div 2-2+BMOfs,7,6);
     if Push=ms_DnBtn then BMOfs:=1 else BMOfs:=0;
     with BtnRect[ms_DnBtn] do
       BMDraw(PaintDC,'MSDN',Right div 2-2,Top+(Bottom-Top) div 2-2+BMOfs,7,6);
     SelectObject(PaintDC,OldPen);
     DeleteObject(GrPen);
     DeleteObject(DkPen);
end;

procedure TMicroScroll.WMNCPaint(var Msg:TMessage);

{-- Draws the border of MicroScroll. The left border is not drawn:
    this avoids overlappings with the right border of the edit ctrl }

var Rect:TRect;
    DC:HDC;

begin
     DC:=GetWindowDC(HWindow);
     GetClientRect(HWindow,Rect);
     with Rect do
       begin
         MoveTo(DC,Left,Top);
         LineTo(DC,Right+1,Top);
         LineTo(DC,Right+1,Bottom+1);
         LineTo(DC,Left,Bottom+1);
       end;
     ReleaseDC(HWindow,DC);
end;

procedure TMicroScroll.WMPaint(var Msg:TMessage);

var PaintDC:HDC;
    PaintStruct:TPaintStruct;

begin
     PaintDC:=BeginPaint(HWindow,PaintStruct);
     Draw(PaintDC,ms_NoBtn,True);
     EndPaint(HWindow,PaintStruct);
end;

procedure TMicroScroll.WMLButtonDblClk(var Msg:TMessage);

{-- Avoids strange behaviour due to default message handling }

begin
     WMLButtonDown(Msg);
end;

procedure TMicroScroll.WMSetFocus(var Msg:TMessage);

begin
     {-- Avoids default message handling: don't remove }
end;

procedure TMicroScroll.WMKillFocus(var Msg:TMessage);

begin
     {-- Avoids default message handling: don't remove }
end;

procedure TMicroScroll.WMTimer(var Msg:TMessage);

{-- Auto-repeat feature }

begin
     if TimeCnt<3 then Inc(TimeCnt)
     else
       if MouseOnBtn then
         SendDlgItemMessage(Parent^.HWindow,AssID,
           wm_User+wm_MicroScroll,BtnPushed,Step);
end;

procedure TMicroScroll.WMLButtonDown(var Msg:TMessage);

var Point:TPoint;
    PaintDC:HDC;
    i:Byte;

begin
     PaintDC:=GetDC(HWindow);
     Point.X:=Msg.lParamLo;
     Point.Y:=Msg.lParamHi;
     BtnPushed:=ms_NoBtn;
     for i:=ms_UpBtn to ms_DnBtn do
       if PtInRect(BtnRect[i],Point) then BtnPushed:=i;
     MouseOnBtn:=BtnPushed<>ms_NoBtn;
     Draw(PaintDC,BtnPushed,False);
     ReleaseDC(HWindow,PaintDC);
     {-- Starts auto-repeat }
     SetTimer(HWindow,1,100,Nil);
     SetCapture(HWindow);
     TimeCnt:=0;
end;

procedure TMicroScroll.WMMouseMove(var Msg:TMessage);

var PaintDC:HDC;
    Point:TPoint;
    MovedOn:Boolean;

begin
     Point.X:=Msg.lParamLo;
     Point.Y:=Msg.lParamHi;
     MovedOn:=PtInRect(BtnRect[BtnPushed],Point);
     if MovedOn<>MouseOnBtn then
       begin
         PaintDC:=GetDC(HWindow);
         if MovedOn then Draw(PaintDC,BtnPushed,False)
                    else Draw(PaintDC,ms_NoBtn,False);
         ReleaseDC(HWindow,PaintDC);
         MouseOnBtn:=MovedOn;
      end;
     DefWndProc(Msg);
end;

procedure TMicroScroll.WMLButtonUp(var Msg:TMessage);

var PaintDC:HDC;

begin
     {-- Stops auto-repeat }
     KillTimer(HWindow,1);
     if MouseOnBtn then
       SendDlgItemMessage(Parent^.HWindow,AssID,
         wm_User+wm_MicroScroll,BtnPushed,Step);
     PaintDC:=GetDC(HWindow);
     Draw(PaintDC,ms_NoBtn,False);
     ReleaseDC(HWindow,PaintDC);
     BtnPushed:=ms_NoBtn;
     MouseOnBtn:=False;
     SetFocus(GetDlgItem(Parent^.HWindow,AssID));
     SetFocus(GetDlgItem(Parent^.HWindow,AssID));
     ReleaseCapture;
end;

end.