(*----------------------------------------------------------------------*
 | SLIDER.PAS                                                           |
 |                                                                      |
 | This example pascal VBX is of a very basic 'slider'.  The slider can |
 | be dragged by moving the mouse - the maximum, minimum & default      |
 | values are VBX properties, as is the knob colour.                    |
 |                                                                      |
 | Programmed by Colin Wilson (woozle@cix) on 19/2/93                   |
 *----------------------------------------------------------------------*)
Library slider;

{$R slider.res}

uses WinTypes, WinProcs, vbapi, resource;

type

{*---------------------------------------------------------------------
 * TSlider is a record containing the instance data for a slider
}

PSlider = ^TSlider;
TSlider = record
  Font : HFONT;                         { Not used                                      }
  Caption : hsz;                        { Data for 'caption' property.  Not displayed   }
  Min, Max, Value : Integer;            { Slider max, min & current value.              }
  Capturing : boolean;                  { True when mouse input is captured             }
  BmpX, BmpY : Integer;                 { Position of 'underneath' bitmap               }
  Underneath : HBitmap                  { Bitmap used to store what was under the knob. }
end;

{*---------------------------------------------------------------------
 * VB properties for a slider.  The first 13 are standard VB properties
 * containing the size, position etc.  The remaining four are our
 * slider properties
}

SliderPropertyTypes =  (
  PR_SLIDER_CTLNAME,                    { Standard VB stuff                             }
  PR_SLIDER_INDEX,
  PR_SLIDER_BACKCOLOR,
  PR_SLIDER_LEFT,
  PR_SLIDER_TOP,
  PR_SLIDER_WIDTH,
  PR_SLIDER_HEIGHT,
  PR_SLIDER_VISIBLE,
  PR_SLIDER_PARENT,
  PR_SLIDER_DRAGMODE,
  PR_SLIDER_DRAGICON,
  PR_SLIDER_TAG,
  PR_SLIDER_HWND,
  PR_SLIDER_CAPTION,                    { Slider caption - not yet displayed            }
  PR_SLIDER_MIN,                        { Minimum value.                                }
  PR_SLIDER_MAX,                        { Maxmimum value.                               }
  PR_SLIDER_VALUE,                      { Current value.                                }
  PR_SLIDER_END                         { Dummy last property.                          }
);

{*---------------------------------------------------------------------
 * Standard VB events. for details see VBAPI.PAS
}

SliderEventTypes = (
  EV_SLIDER_CLICK,
  EV_SLIDER_DRAGDROP,
  EV_SLIDER_DRAGOVER,
  EV_SLIDER_END
);

{*---------------------------------------------------------------------
 * Variables.
}

var
  Bitmap : HBitmap;                     { Knob bitmap, loaded by VBInitCC & freed by VBTERMCC }
  BmWidth, BmHeight : Integer;          { Knob size initialized by  "      }

{*---------------------------------------------------------------------
 * Typed constants (initialised variables!)
}

const
cVBXUser : word = 0;                    { No. of VBX users              }
fDevTimeInited : boolean = False;       { Flag indicates if the VBX has }
                                        { been used in a deveopment     }
                                        { environment                   }

{*---------------------------------------------------------------------
 * 'Caption' property details.  For details see VBAPI.PAS
}
PropertySliderCaption : PROPINFO = (
  npszName     : npChar (PChar ('Caption'));
  fl           : DT_HSZ or PF_fGetData or PF_fSetData or PF_fSetMsg or PF_fSaveData;
  OffsetData   : sizeof (HFont);       { Offset of Caption in TSlider record  }
  infoData     : 0;
  dataDefault  : 0;
  npszEnumList : nNil;
  enumMax      : 0
);

{*---------------------------------------------------------------------
 * 'Min' property details.  For details see VBAPI.PAS
}
PropertySliderMin : PROPINFO = (
  npszName     : npChar (PChar ('Min. value'));
  fl           : DT_SHORT + PF_fGetData + PF_fSetData + PF_fSetMsg + PF_fSaveData;
  OffsetData   : sizeof (HFont) + sizeof (hsz);
                                         { Offset of 'min' in TSlider record }
  infoData     : 0;
  dataDefault  : 0;
  npszEnumList : nNil;
  enumMax      : 0
);

{*---------------------------------------------------------------------
 * 'Max' property details.
}
PropertySliderMax : PROPINFO = (
  npszName     : npChar (PChar ('Max. value'));
  fl           : DT_SHORT + PF_fGetData + PF_fSetData + PF_fSetMsg + PF_fSaveData;
  OffsetData   : sizeof (HFont) + sizeof (hsz) + sizeof (Integer);
  infoData     : 0;
  dataDefault  : 255;
  npszEnumList : nNil;
  enumMax      : 0
);

{*---------------------------------------------------------------------
 * 'Value' property details.
}
PropertySliderValue : PROPINFO = (
  npszName     : npChar (PChar ('Value'));
  fl           : DT_SHORT + PF_fGetData + PF_fSetData + PF_fSetMsg + PF_fSaveData;
  OffsetData   : sizeof (HFont) + sizeof (hsz) + sizeof (Integer) + sizeof (Integer);
  infoData     : 0;
  dataDefault  : 0;
  npszEnumList : nNil;
  enumMax      : 0
);

{*----------------------------------------------------------------------
 * Array of (near) pointers to the properties.  The standard VB ones are
 * special constants defined in VBAPI.pas.  The array must be Nil terminated
}
SliderProperties : array [SliderPropertyTypes] of npPropInfo = (
  PPROPINFO_STD_CTLNAME,
  PPROPINFO_STD_INDEX,
  PPROPINFO_STD_BACKCOLOR,
  PPROPINFO_STD_LEFT,
  PPROPINFO_STD_TOP,
  PPROPINFO_STD_WIDTH,
  PPROPINFO_STD_HEIGHT,
  PPROPINFO_STD_VISIBLE,
  PPROPINFO_STD_PARENT,
  PPROPINFO_STD_DRAGMODE,
  PPROPINFO_STD_DRAGICON,
  PPROPINFO_STD_TAG,
  PPROPINFO_STD_HWND,
  npPropInfo (Ofs (PropertySliderCaption)),
  npPropInfo (Ofs (PropertySliderMin)),
  npPropInfo (Ofs (PropertySliderMax)),
  npPropInfo (Ofs (PropertySliderValue)),
  nNil
);

{*----------------------------------------------------------------------
 * Array of (near) pointers to the events.  These are standard VB ones
 * defined in VBAPI.pas.  The array must be Nil terminated
}
SliderEvents : array [SliderEventTypes] of npEventInfo = (
  PEVENTINFO_STD_CLICK,
  PEVENTINFO_STD_DRAGDROP,
  PEVENTINFO_STD_DRAGOVER,
  nNil
);

{*----------------------------------------------------------------------
 * VB model for a slider.  This is the thing that actually gets registered
 * with VB.  It tells VB where are control function is and what properties
 * and events are defined for our control.
}
modelSlider : MODEL = (
  usVersion           : VB_VERSION;
  fl                  : 0;
  pctlproc            : Nil;    { Pointer to our control function.  Filled in }
                                { in VBINITCC                                 }
  fsClassStyle        : CS_VREDRAW or CS_HREDRAW or CS_BYTEALIGNCLIENT;
  flWndStyle          : 0;
  cbCtlExtra          : sizeof (TSlider);       { Size of our instance data   }
  idBmpPalette        : IDB_SLIDER;
  npszDefCtlName      : npChar (PChar ('Slider'));
  npszClassName       : npChar (PChar ('Slider1'));
  npszParentClassName : nNil;
  npproplist          : Ofs (SliderProperties);         { Our Property list   }
  npeventlist         : Ofs (SliderEvents);             { Our event list      }
  nDefProp            : Byte (PR_SLIDER_BACKCOLOR);
  nDefEvent           : Byte (EV_SLIDER_CLICK);
  nValueProp          : Byte (-1);
  usControlVersion    : 1
);

{*----------------------------------------------------------------------
 * Used by VBGetModelInfo (below).  I haven't experimented with this :
 * it mayn't be used by version 1 controls ??
}

NoModels = 1;
modellistSlider : array [0..NoModels] of lpModel =
(
  @modelSlider,
  Nil
);

modelinfoSlider : ModelInfo =
(
  usVersion : VB100_VERSION;
  lplpModel : @modellistSlider
);

{*----------------------------------------------------------------------
 * Paint the knob.  Erase the previous knob if the Update param is set.
}
procedure UpdateSlider (DC : HDC; var client : TRect; Slider : PSlider; Update : boolean);
var
  OldBmp : HBitmap;
  MemDC  : HDC;
  Pos    : Integer;
begin
  MemDC := CreateCompatibleDC (DC);
  if Slider^.max<= Slider^.min then             { Protect against zero divide }
  begin
    MessageBeep ($ffff);
    Exit
  end;
                                                { Calc. slider posion         }
  pos := client.bottom * Slider^.value div (Slider^.max - Slider^.min);

  if Slider^.Underneath <> 0 then               { Erase previous knob         }
  begin
    OldBmp := SelectObject (MemDC, Slider^.Underneath);
    if Update then BitBlt (DC, Slider^.BmpX, Slider^.BmpY, BmWidth, BmHeight, MemDC, 0, 0, SRCCOPY)
  end
  else
  begin                                         { Create 'underneath' bitmap  }
    Slider^.Underneath := CreateCompatibleBitmap (DC, BmWidth, BmHeight);
    OldBmp := SelectObject (MemDC, Slider^.Underneath)
  end;

                                                { Save bitmap position in     }
                                                { instance data.              }
  Slider^.BmpX := (client.right - BmWidth) div 2;
  Slider^.BmpY := client.bottom - (BmHeight + pos);

                                                { Save 'underneath'           }
  BitBlt (MemDC, 0, 0, BmWidth, BmHeight, DC, Slider^.BmpX, Slider^.BmpY, SRCCOPY);
  SelectObject (MemDC, Bitmap);
                                                { Display the knob.  Use      }
                                                { MERGECOPY to let the colour }
                                                { shine through.              }
  BitBlt (DC, Slider^.BmpX, Slider^.BmpY, BmWidth, BmHeight, MemDC, 0, 0, MERGECOPY);

  SelectObject (MemDC, OldBmp);                 { Tidy up...                  }
  DeleteObject (MemDC)
end;

{*----------------------------------------------------------------------
 * Paint the whole slider & surround
}
procedure PaintSlider (SLIDER : pSlider; Wnd : HWnd; DC : HDC);
var
  Client : TRect;
  TheBrush, OldBrush : HBrush;
begin
  GetClientRect (Wnd, client);                  { Get control colour -  this  }
                                                { is set in our BACKCOLOR     }
                                                { property.                   }
  TheBrush := SendMessage (GetParent (Wnd), WM_CTLCOLOR, DC, MAKELONG (Wnd, 0));
  if TheBrush <> 0 then OldBrush := SelectObject (DC, TheBrush) else OldBrush := 0;

                                                { Draw the slide line.        }
  MoveTo (DC, client.right div 2, client.top);
  LineTo (DC, client.right div 2, client.bottom);

                                                { Draw the knob.              }
  UpdateSlider (DC, client, Slider, False);
  if OldBrush <> 0 then SelectObject (DC, OldBrush)
end;

{*----------------------------------------------------------------------
 * Paint the knob only.  Use by WM_MOUSEMOVE
}
procedure RepaintKnob (Wnd : hWnd; Slider : PSlider);
var
  client : TRect;
  DC : HDC;
  TheBrush, OldBrush : HBrush;
begin
  DC := GetDC (Wnd);
  GetClientRect (Wnd, client);
                                                { Get control colour          }
  TheBrush := SendMessage (GetParent (Wnd), WM_CTLCOLOR, DC, MAKELONG (Wnd, 0));
  if TheBrush <> 0 then OldBrush := SelectObject (DC, TheBrush) else OldBrush := 0;

  UpdateSlider (DC, client, Slider, TRUE);
  if OldBrush <> 0 then SelectObject (DC, OldBrush);
  ReleaseDC (Wnd, DC)
end;


{*----------------------------------------------------------------------
 * Return true if point P is on the slider knob (not!)
}
function InKnob (SLIDER : pSlider; p : TPoint) : boolean;
begin
  InKnob := True
end;

{*----------------------------------------------------------------------
 * The control function.  This is called by VB so must be declared
 * 'export'.  Note that the default VB handler is called at the end,
 * unless 'exit' is called as in WM_PAINT.
}
function SliderCtrlProc (Ctl : HCtl; Wnd : hWnd; Msg, wParam : word; lParam :longint) : longint; export;
var
  rc : LongInt;
  Slider : pSlider;
  ps : TPaintStruct;
  Client : TRect;
begin
  Slider := VBDerefControl (Ctl);               { Get slider instance data }
  SliderCtrlProc := 0;

  case Msg of
    WM_PAINT :                                  { Paint the control.  if   }
      begin                                     { wParam <> 0, it contains }
        if wParam <> 0 then                     { a valid DC.              }
          PaintSlider (Slider, Wnd, HDC (wParam))
        else
        begin
          BeginPaint (Wnd, ps);
          PaintSlider (Slider, Wnd, ps.hdc);
          EndPaint (Wnd, ps)
        end;
        exit
      end;

    VBM_SETPROPERTY:                            { Property details changed }
      case SliderPropertyTypes (wParam) of
        PR_SLIDER_CAPTION,                      { If these change, redisp. }
        PR_SLIDER_MIN,                          { the whole gaboosh.       }
        PR_SLIDER_MAX :
          begin
	    InvalidateRect(Wnd, Nil, TRUE);
            Exit
          end;

        PR_SLIDER_VALUE:                        { If the value changes,   }
        begin                                   { just redisplay the knob.}
          RepaintKnob (Wnd, Slider);
          Exit
        end
      end;

    WM_NCCREATE:                                { Initialise instance     }
      begin
        Slider^.Capturing := FALSE;
        Slider^.Underneath := 0;
        VBSetControlProperty (Ctl, Word (PR_SLIDER_MAX), 255);
      end;

    WM_LBUTTONDOWN:                             { Capture mouse input     }
      if (InKnob (Slider, MAKEPOINT (lParam))) then
      begin
        Slider^.Capturing := TRUE;
        SetCapture (Wnd);
      end;

    WM_LBUTTONUP:                               { Release mouse input     }
      if Slider^.Capturing then
      begin
        Slider^.Capturing := FALSE;
        ReleaseCapture
      end;

    WM_MOUSEMOVE:              { Mouse moved.  Using VBSetControlProperty }
                               { will cause a VBM_SETPROPERTY message to  }
                               { be sent, which updates the knob posn.    }

      if Slider^.Capturing then
      begin
        GetClientRect (Wnd, client);
        VBSetControlProperty (Ctl, Word (PR_SLIDER_VALUE),
                              (Slider^.max - Slider^.min) * (client.bottom - HIWORD (lParam)) div client.bottom);
      end;

    WM_NCDESTROY:              { Tidy up on deletion.                     }
      begin
        if Slider^.Caption <> Nil then VBDestroyHsz (Slider^.Caption);
        if Slider^.Underneath <> 0 then DeleteObject (Slider^.Underneath)
      end
  end;

  SliderCtrlProc := VBDefControlProc (Ctl, Wnd, Msg, wParam, lParam);
end;

{*----------------------------------------------------------------------
 * Initialise the VBX.  Register our control with VB.  Called by VB, so
 * must be 'export'
}
function VBInitCC (Version : word; fRuntime : boolean) : boolean; export;
var
  bm : TBitmap;
begin
  modelSlider.pctlproc := @SliderCtrlProc;  { Fill in control func. address }

  if cVbxUser = 0 then                      { First user ??                 }
  begin
                                            { Load the bitmap.              }
    Bitmap := LoadBitmap (hInstance, MakeIntResource (IDB_SLIDERKNOB));
    GetObject (Bitmap, sizeof(TBitmap), @bm);
    BmWidth := bm.bmWidth;
    BmHeight := bm.bmHeight
  end;

  Inc (cVbxUser);

  if (not fRuntime) and (not fDevTimeInited) then
  begin
    fDevTimeInited := TRUE;

    { Development system initilised.  You can initialise flash dialog boxes     }
    { for property details entry here.                                          }
  end;

                                           { Register with VB                   }
  VBInitCC := VBRegisterModel(hInstance, modelSlider)
end;

{*----------------------------------------------------------------------
 * Destroy the VBX.  Called by VB, so must be export
}
procedure VBTermCC; export;
begin
  Dec (cVbxUser);
  if cVbxUser = 0 then
  begin
    DeleteObject (Bitmap);                { Last user.  Tidy up.                }
    if fDevTimeInited then
    begin
                                          { Tidy up if development environment  }
    end
  end
end;

{*----------------------------------------------------------------------
 * Return the model info to VB.  Must be export
}
function VBGetModelInfo (Version : word) : lpModelInfo; export;
begin
  VBGetModelInfo := @modelinfoSlider
end;

EXPORTS
  VBInitCC index 1,
  VBTermCC index 2,
  VBGetModelInfo index 3;

begin
end.