{WinPCX demo program
CopyRight 1993, Mark D.Rafn, MDRUtils(tm)}
unit uCanvas;
interface
uses OWindows, WinTypes, WinProcs;

type

	PState = ^TState;
  TState = record
  	OriginalSize: TPoint;
  	BitMapSize: TPoint;
    Zoom: boolean;
  end;

	PCanvas = ^TCanvas;
  TCanvas = object(TWindow)
  	State: PState;
  	Bitmap: HBitMap;
    Palette: HPalette;
    Location: TPoint;
    constructor Init(AParent: PWindowsObject; AState: PState);
    destructor Done; virtual;
    procedure Paint(PaintDC: HDC; var PaintInfo: TPaintStruct); virtual;
		procedure Moveself(wx, wy, ww, wh: integer; Repaint: boolean);
    function LoadFile(Filename: PChar): integer;
  end;

  PCanvasScroller = ^TCanvasScroller;
  TCanvasScroller = object(TScroller)
  	procedure BeginView(PaintDC: HDC; var PaintInfo: TPaintStruct); virtual;
  end;

implementation
uses uPcxwin;

constructor TCanvas.Init(AParent: PWindowsObject; AState: PState);
var
	DC: HDC;
begin
	TWindow.Init(AParent, nil);
  Attr.Style := ws_Border or ws_Child or ws_Visible or
		ws_HScroll or ws_VScroll;
  Scroller := New(PCanvasScroller, Init(@self, 1,1,200,200));
  State := AState;
end;

destructor TCanvas.Done;
begin
	if Bitmap <> 0 then DeleteObject(Bitmap);
  if Palette <> 0 then DeleteObject(Palette);
  TWindow.Done;
end;

procedure TCanvas.Paint(PaintDC: HDC; var PaintInfo: TPaintStruct);
var
  OldPal: THandle;
  MemDC: HDC;
  BMInfo: TBitmap;
begin
  GetObject(Bitmap, sizeof(TBitmap), @BMInfo);
  MemDC := CreateCompatibleDC(PaintDC);
  SelectObject(MemDC, Bitmap);
	OldPal := SelectPalette(MemDC, Palette, false);
  UnRealizeObject(Palette);
  RealizePalette(MemDC);
  with State^.BitmapSize, BMInfo do
  if State^.Zoom then
  	StretchBlt(PaintDC, 0, 0, x, y, MemDC, 0, 0, bmWidth, bmHeight, SrcCopy)
  else
	  BitBlt(PaintDC, 0, 0, x, y, MemDC, 0, 0, SrcCopy);
  {select Palette out of the device context}
  SelectObject(MemDC, OldPal);
  DeleteDC(MemDC);
end;

function TCanvas.LoadFile(Filename: PChar): integer;
var
  Pcx: PPcxWin;
  BM: TBitMap;
  HBM: HBitmap;
begin
	LoadFile := 0;
	Pcx := New(PPcxwin, Init(Filename));
  if Assigned(Pcx) then
  	Pcx^.LoadPCXBitMap(HBM, Palette)
  else Exit;
  GetObject(HBM, sizeof(BM), @BM);
  with State^.BitmapSize, BM do
  begin
  	x := bmWidth;
    y := bmHeight;
  end;
  State^.OriginalSize := State^.BitmapSize;
  Bitmap := HBM;
  Dispose(Pcx, Done);
  LoadFile := 1;
end;

procedure TCanvas.Moveself(wx, wy, ww, wh: integer; Repaint: boolean);
var
	XRange, YRange: integer;
begin
	with State^.BitMapSize do
  begin
  	if ww > x then
    begin
    	XRange := 0;
      ww := x;
    end
    else
    	XRange := x - ww;
    if wh > y then
    begin
    	YRange := 0;
      wh := y;
    end
    else
    	YRange := y - wh;
  end;
  if (Attr.x = wx) and (Attr.y = wy) and (Attr.w = ww) and
  	(Attr.h = wh) and Repaint then
    InvalidateRect(HWindow, nil, true)
  else
  	MoveWindow(HWindow, wx, wy, ww, wh, Repaint);
  Scroller^.SetRange(XRange, Scroller^.YRange);
  Scroller^.SetRange(Scroller^.XRange, YRange);
  Scroller^.ScrollTo(0, 0);
end;

procedure TCanvasScroller.BeginView(PaintDC: HDC; var PaintInfo: TPaintStruct);
begin
	TScroller.BeginView(PaintDC, PaintInfo);

end;

end.