{Copyright 1993, Mark D. Rafn, MDRUtils(tm)}
{Target: Windows 3.1}
{$R-}
unit UPcxWin;
interface
uses WinTypes,WinProcs, Objects, BWCC;
type

	{used this structure instead of TRGBTriple.}
  {TRGBTriple is reversed, confusing with read results}
	rRgb = record
		r, g, b: byte;
  end;

  PPcxPal_256 = ^PcxPal_256;
  PPcxPal_16 = ^PcxPal_16;
  PcxPal_256 = array[0..255] of rRGB;
  PcxPal_16 = array[0..15] of rRGB;

	LongType = record
		case Word of
			0: (Ptr: pointer);
			1: (Long: Longint);
			2: (Lo: word; Hi: word);
	end;

	PHeader = ^PCXHeader;
	PCXHeader = record
		Maker:				byte;
		Version:			byte;
		Encoding:			byte;
		Bpp: 					byte;
		Xmn, Ymn:			integer;
		Xmx, Ymx:			integer;
		HRes, VRes: 	integer;
		CMap: 				PcxPal_16;
		Reserved :		byte;
		NPlanes: 			byte;
		NBpl:					integer;
		PalType: 			integer;
	end;

  { *The* object for decoding a PCX format image}
  { Note: this is not a window, the name is just consistant}
  { with previous naming conventions. ie: MPcxVga }
  { The object's function is pass back a bitmap and palette handle.}
  { What you do with it from there is up to you. }
	PPcxWin = ^MPcxWin;
	MPcxWin = object(TObject)
		Header: PHeader;
		Hf: integer;
		constructor Init(Filename: PChar);
		destructor Done; virtual;
    function GetHeaderInfo: pointer;
		procedure LoadPCXBitMap(var HBits: hBitmap; var HPal: hPalette);
	end;

  {Support objects for MPcxWin}
	PDecoder = ^MDecoder;
	MDecoder = object
    BufferIndex: integer;
		BytesPerLine: integer;
    FBuffer: array[0..1023] of byte;
		Hdr: PHeader;
    Height: longint;
    HF: integer;
		Palette: HPalette;
		Width: longint;
		constructor Init(F: integer; Header: PHeader);
		destructor Done; virtual;
		function CreateDIB(var DIBPal: hPalette): HBitmap; virtual;
		procedure Decode(IMem: THandle; ISize: longint; Bpl: longint); virtual;
		procedure Decode_Row(var S: pointer); virtual;
    function GetBMI(var Size: word): PBitMapInfo; virtual;
		function GetPaletteInfo: pointer; virtual;
    procedure CreatePCXPalette(BMI: PBitMapInfo); virtual;
	end;

	PDecoder16 = ^MDecoder16;
	MDecoder16 = object(MDecoder)
    procedure CreatePCXPalette(BMI: PBitMapInfo); virtual;
		procedure Decode(IMem: THandle; ISize: longint; Bpl: longint); virtual;
    function GetBMI(var Size: word): PBitMapInfo; virtual;
		procedure Convert_Row(Bpl: longint; Sourc: pointer; var Dest: pointer);
	end;

	PDecoder256 = ^MDecoder256;
	MDecoder256 = object(MDecoder)
    procedure CreatePCXPalette(BMI: PBitMapInfo); virtual;
		procedure Decode(IMem: THandle; ISize: longint; Bpl: longint); virtual;
    function GetBMI(var Size: word): PBitMapInfo; virtual;
		function GetPaletteInfo: pointer; virtual;
	end;

const
	PCXError0 = 'OK';
	PCXError1 = 'Unable to open file';
  PCXError2 = 'File read error!';
	PCXError3 = 'Not a PCX file!';
  PCXError4 = 'Unsupported PCX file format.';
	PCXError5 = 'Out of Memory';
	PCXError6 = 'Cannot create 256 color palette';
  PCXError7 = 'DIB Creation Error';

var
	PCXError: PChar;

procedure PCXErrorMessage(Parent: HWnd);

implementation

procedure AHIncr; far; external 'KERNEL' index 114;

function AlignDouble(Size: longint): longint;
begin	AlignDouble := (Size + 3) div 4 * 4;	end;

procedure PCXErrorMessage(Parent: HWnd);
begin	BWCCMessageBox(Parent, PCXError, 'Error', mb_OK + mb_IconExclamation); end;

{---------- MPcxWin Methods --------}

constructor MPcxWin.Init(Filename: PChar);
begin
	inherited Init;
  PCXError := PCXError1;
	Hf := _LOpen(Filename, of_Read);
	if Hf = -1 then	Fail;

	PCXError := PCXError2;
	Header := PHeader(GetHeaderInfo);
	if Header = nil then Fail;

	PCXError := PCXError3;
	if Header^.Maker <> $0A then Fail;
	PCXError := PCXError0;
end;

destructor MPcxWin.Done;
begin
	Dispose(Header);
	_lClose(Hf);
end;

{Get PCX header info for format validation}
function MPcxWin.GetHeaderInfo: pointer;
var
  Hdr: PHeader;
begin
	New(Hdr);
  _llSeek(Hf, 0, 0);
	if (_lRead(Hf, @Hdr^, Sizeof(PCXHeader)) <> Sizeof(PCXHeader)) then
		Dispose(Hdr);
	GetHeaderInfo := Hdr;
end;

{Initialize correct decoder instance, decode and return DIB and Palette handles }
procedure MPcxWin.LoadPCXBitMap(var hBits: HBitmap; var hPal: hPalette);
var
	Decoder: PDecoder;
  HDIB: HBitmap;
begin
	PCXError := PcxError2;
	if (Header^.Bpp = 8) and (Header^.NPlanes = 1) then
		Decoder := New(PDecoder256, Init(HF, Header))
	else if (Header^.Bpp = 1) and (Header^.NPlanes = 4) then
    Decoder := New(PDecoder16, Init(HF, Header))
  else
  begin
  	PcxError := PcxError3;
    Exit;
  end;
	if not Assigned(Decoder) then	Exit;
  PCXError := PcxError7;
	HDIB := Decoder^.CreateDIB(hPal);
	if HDIB = 0 then Exit;
	hBits := HDIB;
  Dispose(Decoder, Done);
end;

{---------- MDecoder Methods --------}

constructor MDecoder.Init(F: integer; Header: PHeader);
begin
	HF := F;
	Hdr := Header;
	_llseek(hF, 128, 0);
	Palette := 0;
	BufferIndex := 0;
	BytesPerLine := Hdr^.NBpl;
  Width := longint(Hdr^.Xmx) - longint(Hdr^.Xmn) + 1;
  Height := longint(Hdr^.Ymx) - longint(Hdr^.Ymn) + 1;
end;

destructor MDecoder.Done; begin end;

function MDecoder.CreateDIB(var DIBPal: hPalette): HBitmap;
var
	BMInfo: PBitMapInfo;
	bmiSize: word;
	DCHandle: HDC;
	DIBBytesPerLine: longint;
	HImage,
	HImageNew: HBitmap;
	i: integer;
	ImageSize: longint;
	PImage: pointer;
  OldPal: hPalette;

begin
	CreateDIB := 0;
  if (Hdr^.Bpp = 1) and (Hdr^.NPlanes = 4) then
  	DIBBytesPerLine := AlignDouble(longint(Width) div 2)
  else DIBBytesPerLine := AlignDouble(BytesPerLine);
	ImageSize := DIBBytesPerLine * Height;
	GlobalCompact(ImageSize);
	HImage := GlobalAlloc(gmem_Moveable or gmem_ZeroInit, ImageSize);
	if HImage = 0 then Exit;
	Decode(HImage, ImageSize, DIBBytesPerLine);
  BMInfo := GetBMI(bmiSize);
  CreatePCXPalette(BMInfo);
	DCHandle := GetDC(0);
	PImage := GlobalLock(HImage);
  OldPal := SelectPalette(DCHandle, Palette, false);
  UnRealizeObject(Palette);
  RealizePalette(DCHandle);
	HImageNew := CreateDIBitmap(DCHandle, BMInfo^.bmiHeader, cbm_Init,
		PImage, BMInfo^, 0);
  SelectPalette(DCHandle, OldPal, false);
	ReleaseDC(0, DCHandle);
  GlobalUnlock(HImage);
  Globalfree(HImage);
  FreeMem(BMInfo, bmiSize);
  DIBPal := Palette;
  CreateDIB := HImageNew;
end;

{ Decode an entire scanline into S regardless of image type }
procedure MDecoder.Decode_Row(var S: pointer);
var
  i, ByteCount, Repeats, RunLength, Plane, NoRead: integer;
	b: byte;
  SAddr: LongType;
  StartOfs: longint;
  NumRead: integer;
begin
  SAddr.Ptr := S;
  StartOfs := SAddr.Lo;
  ByteCount := 0;
  RunLength := Hdr^.NBpl * Hdr^.NPlanes;
  if BufferIndex = 0 then
    NumRead := _lread(HF, @FBuffer, Sizeof(FBuffer));
  while ( ByteCount < RunLength ) do
  begin
    if BufferIndex = 1024 then
    begin
      NumRead := _lread(HF, @FBuffer, Sizeof(FBuffer));
      BufferIndex := 0;
    end;
    b := FBuffer[BufferIndex];
    BufferIndex := BufferIndex + 1;
   if ( b >= 192 ) then
   begin
     Repeats := b - 192;
     if BufferIndex = 1024 then
     begin
       NumRead := _lread(HF, @FBuffer, Sizeof(FBuffer));
       BufferIndex := 0;
     end;
     b := FBuffer[BufferIndex];
     BufferIndex := BufferIndex + 1;
     for i :=  1 to Repeats do
     begin
       Mem[SAddr.Hi:SAddr.Lo] := b;
       ByteCount := ByteCount + 1;
       SAddr.Lo := StartOfs + ByteCount;
     end;
   end
   else
   begin
     Mem[SAddr.Hi:SAddr.Lo] := b;
     ByteCount := ByteCount + 1;
     SAddr.Lo := StartOfs + ByteCount;
   end;
	end;
end;

{Empty methods}
procedure MDecoder.Decode(IMem: THandle; ISize: longint; Bpl: longint); begin end;
function MDecoder.GetPaletteInfo: pointer; begin end;
function MDecoder.GetBMI(var Size: word): PBitMapInfo; begin end;
procedure MDecoder.CreatePCXPalette(BMI: PBitMapInfo); begin end;

{---------- MDecoder256 Methods --------}

procedure MDecoder256.Decode(IMem: THandle; ISize: longint; Bpl: longint);
var
	Start, ToAddr, Bits, Source: LongType;
	ScanLine: pointer;
  LineNo, i: integer;
begin
	Bits.Ptr := GlobalLock(IMem);
	GetMem(ScanLine, Bpl);
  if _llseek(hF, 128, 0) = -1 then Exit;
	Source.Ptr := ScanLine;
	for LineNo := (Height - 1) downto 0 do
	begin
		Decode_Row(ScanLine);
		Start.Long := longint(LineNo) * Bpl;
		Source.Ptr := ScanLine;
		for i := 1 to Bpl do
		begin
			ToAddr.Hi := Bits.Hi + (Start.Hi * Ofs(AHIncr));
			ToAddr.Lo := Start.Lo;
			Mem[ToAddr.Hi:ToAddr.Lo] := Mem[Source.Hi:Source.Lo];
			Source.Long := Source.Long + 1;
			Start.Long := Start.Long + 1;
		end;
	end;
	FreeMem(ScanLine, Bpl);
  GlobalUnLock(IMem);
end;

function MDecoder256.GetBMI(var Size: word): PBitMapInfo;
var
	BitMapInfo: PBitMapInfo;
  i: integer;
	bmiSize: word;
  PalInfo: PPcxPal_256;
begin
  bmiSize := SizeOf(TBitmapInfoHeader) + (Sizeof(TRGBQuad) * 256);
	GetMem(BitMapInfo, bmiSize);
	BitmapInfo^.bmiHeader.biSize := Sizeof(TBitmapInfoHeader);
	BitmapInfo^.bmiHeader.biWidth := Width;
	BitmapInfo^.bmiHeader.biHeight := Height;
	BitmapInfo^.bmiHeader.biPlanes := 1;
	BitmapInfo^.bmiHeader.biBitCount := 8;
	BitmapInfo^.bmiHeader.biCompression := 0;
	BitmapInfo^.bmiHeader.biSizeImage := 0;
	BitmapInfo^.bmiHeader.biXPelsperMeter := 0;
	BitmapInfo^.bmiHeader.biYPelsperMeter := 0;
	BitmapInfo^.bmiHeader.biClrUsed := 256;
	BitmapInfo^.bmiHeader.biClrImportant := 0;
	PalInfo := PPcxPal_256(GetPaletteInfo);
	if Assigned(PalInfo) then
  begin
  	for i := 0 to 255 do
    	with BitMapInfo^.bmiColors[i], PalInfo^[i] do
      begin
			  rgbRed := r;
			  rgbGreen := g;
			  rgbBlue := b;
			  rgbReserved := 0;
  	  end;
		FreeMem(PalInfo, Sizeof(PcxPal_256));
  end;
  Size := bmiSize;
  GetBMI := BitMapInfo;
end;

procedure MDecoder256.CreatePCXPalette(BMI: PBitMapInfo);
var
	LogPalette: PLogPalette;
  i: integer;
  PalSize: word;
begin
  if Palette <> 0 then
  begin
  	DeleteObject(Palette);
    Palette := 0;
  end;
	PalSize := Sizeof(TLogPalette) + (256 * Sizeof(TPaletteEntry)); {check this size?}
  GetMem(LogPalette, PalSize);
  for i := 0 to 255 do
  	with LogPalette^ do
    begin
    	palNumEntries := 256;
      palVersion := $300;
      with palPalEntry[i], BMI^.bmiColors[i] do
      begin
      	peRed := rgbRed;
        peGreen := rgbGreen;
        peBlue := rgbBlue;
        peFlags := 0;
      end;
    end;
  Palette := CreatePalette(LogPalette^);
  FreeMem(LogPalette, PalSize);
end;

function MDecoder256.GetPaletteInfo: pointer;
var
	TempPal: PPcxPal_256;
	i: integer;
	b: byte;
begin
  GetPaletteInfo := nil;
	if (_llseek(hF, -769, 2)) = -1 then Exit;
  if ((_lread(hF, @b, 1)) <> -1) then
		if b = $0C then
		begin { 256k palette exists }
      New(TempPal);
			if (_lread(hF, @TempPal^, Sizeof(PcxPal_256))) = -1 then
      	Dispose(TempPal)
      else
      	GetPaletteInfo := TempPal;
    end;
end;

{---------- MDecoder16 Methods --------}

procedure MDecoder16.Decode(IMem: THandle; ISize: longint; Bpl: longint);
var
	Start, ToAddr, Bits, Source: LongType;
	ScanLine: pointer;
  LineNo, i: integer;
begin
	Bits.Ptr := GlobalLock(IMem);
	GetMem(ScanLine, Bpl);
  if _llseek(hF, 128, 0) = -1 then Exit;
	Source.Ptr := ScanLine;
	for LineNo := (Height - 1) downto 0 do
	begin
		Decode_Row(ScanLine);
    Convert_Row(Bpl, ScanLine, ScanLine);
		Start.Long := longint(LineNo) * Bpl;
		Source.Ptr := ScanLine;
		for i := 1 to Bpl do
		begin
			ToAddr.Hi := Bits.Hi + (Start.Hi * Ofs(AHIncr));
			ToAddr.Lo := Start.Lo;
			Mem[ToAddr.Hi:ToAddr.Lo] := Mem[Source.Hi:Source.Lo];
			Source.Long := Source.Long + 1;
			Start.Long := Start.Long + 1;
		end;
	end;
	FreeMem(ScanLine, Bpl);
  GlobalUnLock(IMem);
end;

function MDecoder16.GetBMI(var Size: word): PBitMapInfo;
var
	BitMapInfo: PBitMapInfo;
  i: integer;
	bmiSize: word;
begin
  bmiSize := SizeOf(TBitmapInfoHeader) + (Sizeof(TRGBQuad) * 16);
	GetMem(BitMapInfo, bmiSize);
	BitmapInfo^.bmiHeader.biSize := Sizeof(TBitmapInfoHeader);
	BitmapInfo^.bmiHeader.biWidth := Width;
	BitmapInfo^.bmiHeader.biHeight := Height;
	BitmapInfo^.bmiHeader.biPlanes := 1;
	BitmapInfo^.bmiHeader.biBitCount := 4;
	BitmapInfo^.bmiHeader.biCompression := 0;
	BitmapInfo^.bmiHeader.biSizeImage := 0;
	BitmapInfo^.bmiHeader.biXPelsperMeter := 0;
	BitmapInfo^.bmiHeader.biYPelsperMeter := 0;
	BitmapInfo^.bmiHeader.biClrUsed := 16;
	BitmapInfo^.bmiHeader.biClrImportant := 0;
  for i := 0 to 15 do
    with BitMapInfo^.bmiColors[i], Hdr^.CMap[i] do
    begin
			rgbRed := r;
			rgbGreen := g;
			rgbBlue := b;
			rgbReserved := 0;
  	end;
  Size := bmiSize;
  GetBMI := BitMapInfo;
end;

procedure MDecoder16.CreatePCXPalette(BMI: PBitMapInfo);
var
	LogPalette: PLogPalette;
  i: integer;
  PalSize: word;
begin
  if Palette <> 0 then
  begin
  	DeleteObject(Palette);
    Palette := 0;
  end;
	PalSize := Sizeof(TLogPalette) + (16 * Sizeof(TPaletteEntry)); {check this size?}
  GetMem(LogPalette, PalSize);
  for i := 0 to 15 do
  	with LogPalette^ do
    begin
    	palNumEntries := 16;
      palVersion := $300;
      with palPalEntry[i], BMI^.bmiColors[i] do
      begin
      	peRed := rgbRed;
        peGreen := rgbGreen;
        peBlue := rgbBlue;
        peFlags := 0;
      end;
    end;
  Palette := CreatePalette(LogPalette^);
  FreeMem(LogPalette, PalSize);
end;

procedure MDecoder16.Convert_Row(Bpl: longint; Sourc: pointer; var Dest: pointer);
var
	Nibbles: byte;
	Start, ToAddr, Bits, Source: LongType;
  r, g, b, i, j, k: byte;
begin
	Source.Ptr := Sourc;
	GetMem(Bits.Ptr, Bpl);
  Start.Long := Bits.Long;
  for j := 1 to BytesperLine do
  begin
    r := Mem[Source.Hi:Source.Lo];
    g := Mem[Source.Hi:Source.Lo + BytesPerLine];
    b := Mem[Source.Hi:Source.Lo + (BytesperLine * 2)];
    i := Mem[Source.Hi:Source.Lo + (BytesperLine * 3)];
    for k := 0 to 3 do
    begin
      Nibbles := 0;
      if ((r and $80) = $80) then Nibbles := Nibbles or $10;
      if ((g and $80) = $80) then Nibbles := Nibbles or $20;
      if ((b and $80) = $80) then Nibbles := Nibbles or $40;
      if ((i and $80) = $80) then Nibbles := Nibbles or $80;
      r := r shl 1; g := g shl 1; b := b shl 1; i := i shl 1;
      if ((r and $80) = $80) then Nibbles := Nibbles or $01;
      if ((g and $80) = $80) then Nibbles := Nibbles or $02;
      if ((b and $80) = $80) then Nibbles := Nibbles or $04;
      if ((i and $80) = $80) then Nibbles := Nibbles or $08;
      r := r shl 1; g := g shl 1; b := b shl 1; i := i shl 1;
      ToAddr.Hi := Bits.Hi;
      ToAddr.Lo := Start.Lo;
      Mem[ToAddr.Hi:ToAddr.Lo] := Nibbles;
      Start.Long := Start.Long + 1;
    end;
    Source.Long := Source.Long + 1;
  end;
  FreeMem(Sourc, Bpl);
  Dest := Bits.Ptr;
end;


end.
