
unit VBApi;

{---------------------------------------------------------------------------}
{ This routine provides TPW programs with an interface to the routines      }
{ needed to create a Visual Basic custom control (VBX) DLL.  A couple of    }
{ points:                                                                   }
{                                                                           }
{ (1).  The interface described here is for VB 1.0.  My understanding is    }
{       that there are new calls/structures in VB 2.0.  Also, I have not    }
{       finished defining all the needed structures required by the         }
{       interface. Routines that take not-yet-defined structure pointers    }
{       simply declares as taking pointers.  If you get further with this,  }
{       or find any errors, then please keep me updated.                    }
{                                                                           }
{ (2).  The code is completely untested, although I think it stands a good  }
{       chance of working.  The underlying assumption (based on looking at  }
{       Microsfoft's code) is that VB installs a far procedure pointer at   }
{       absolute location $20 in the stack segment.  This makes sense (and  }
{       is actually quite ingenious) because since a VBX is a DLL, it uses  }
{       the caller's stack segment.  The caller in this case is VB and it   }
{       can obviously do what it likes with its own stack segment.  In      }
{       effect, it constitutes a private, hidden i/f between the DLL and    }
{       it's caller.  Presumably, the handler in VB switches on the value   }
{       in the BL register and picks up the on-stack arguments depending on }
{       this value.                                                         }
{                                                                           }
{ (3).  Ok, three points....!  If you get as far as building a working VBX  }
{       with this code, I would love to know.  I spent an evening playing   }
{       with this when I was supposed to be conversing with the wife/kids   }
{       and would like to know it wasn't time wasted.                       }
{                                                                           }
{ (4).  Voila! A working VBX.                                               }
{                               Dave Jewell, djewell@cix, fisherman@bix     }
{                               Colin Wilson, woozle@cix                    }
{---------------------------------------------------------------------------}

interface

uses WinTypes, WinProcs;

type
    HCtl = Pointer;
    HSz = PChar;
    HLStr = Pointer;
    npWord = word;
    npChar = word;
    npPropInfo = word;
    npEventInfo = word;

const
  VB100_VERSION = $0100;
  VB200_VERSION = $0200;
  VB300_VERSION = $0300;
  VB_VERSION    = VB100_VERSION;

  nNil          = 0;
  DT_HSZ        = $01;
  DT_SHORT      = $02;
  DT_LONG       = $03;
  DT_BOOL       = $04;
  DT_COLOR      = $05;
  DT_ENUM       = $06;
  DT_REAL       = $07;
  DT_XPOS       = $08;
  DT_XSIZE      = $09;
  DT_YPOS       = $0A;
  DT_YSIZE      = $0B;
  DT_PICTURE    = $0C;
  DT_HLSTR      = $0D;
  DT_OBJECT     = $20;

  PF_datatype       = $000000FF;
  PF_fPropArray     = $00000100;
  PF_fSetData       = $00000200;
  PF_fSetMsg        = $00000400;
  PF_fNoShow        = $00000800;
  PF_fNoRuntimeW    = $00001000;
  PF_fGetData       = $00002000;
  PF_fGetMsg        = $00004000;
  PF_fSetCheck      = $00008000;
  PF_fSaveData      = $00010000;
  PF_fSaveMsg       = $00020000;
  PF_fLoadDataOnly  = $20010000;
  PF_fLoadMsgOnly   = $20020000;
  PF_fGetHszMsg     = $00040000;
  PF_fUpdateOnEdit  = $00080000;
  PF_fEditable      = $00100000;
  PF_fPreHwnd       = $00200000;
  PF_fDefVal        = $00400000;
  PF_fNoInitDef     = $00800000;
  PF_fNoRuntimeR    = $02000000;
  PF_fNoMultiSelect = $04000000;

(*---------------------------------------------------------------------------
// Standard control properties
//---------------------------------------------------------------------------*)
  IPROP_STD_NAME           = $0000;
  IPROP_STD_CTLNAME        = IPROP_STD_NAME;
  IPROP_STD_INDEX          = $0001;
  IPROP_STD_HWND           = $0002;
  IPROP_STD_BACKCOLOR      = $0003;
  IPROP_STD_FORECOLOR      = $0004;
  IPROP_STD_LEFT           = $0005;
  IPROP_STD_TOP            = $0006;
  IPROP_STD_WIDTH          = $0007;
  IPROP_STD_HEIGHT         = $0008;
  IPROP_STD_ENABLED        = $0009;
  IPROP_STD_VISIBLE        = $000A;
  IPROP_STD_MOUSEPOINTER   = $000B;
  IPROP_STD_CAPTION        = $000C;
  IPROP_STD_FONTNAME       = $000D;
  IPROP_STD_FONTBOLD       = $000E;
  IPROP_STD_FONTITALIC     = $000F;
  IPROP_STD_FONTSTRIKE     = $0010;
  IPROP_STD_FONTUNDER      = $0011;
  IPROP_STD_FONTSIZE       = $0012;
  IPROP_STD_TABINDEX       = $0013;
  IPROP_STD_PARENT         = $0014;
  IPROP_STD_DRAGMODE       = $0015;
  IPROP_STD_DRAGICON       = $0016;
  IPROP_STD_BORDERSTYLEOFF = $0017;
  IPROP_STD_TABSTOP        = $0018;
  IPROP_STD_TAG            = $0019;
  IPROP_STD_TEXT           = $001B;
  IPROP_STD_BORDERSTYLEON  = $001C;
  IPROP_STD_CLIPCONTROLS   = $001D;
  IPROP_STD_NONE           = $001E;
  IPROP_STD_HELPCONTEXTID  = $001F;
  IPROP_STD_LINKMODE       = $0020;
  IPROP_STD_LINKITEM       = $0021;
  IPROP_STD_LINKTOPIC      = $0022;
  IPROP_STD_LINKTIMEOUT    = $0023;
  IPROP_STD_LEFTNORUN      = $0024;
  IPROP_STD_TOPNORUN       = $0025;
  IPROP_STD_ALIGN          = $0026;
  IPROP_STD_IMEMODE        = $0027;
  IPROP_STD_DATASOURCE     = $0028;
  IPROP_STD_DATAFIELD      = $0029;
  IPROP_STD_DATACHANGED    = $002A;
  IPROP_STD_LAST           = $0FFF;

  PPROPINFO_STD_NAME           = npPropInfo (not IPROP_STD_NAME);
  PPROPINFO_STD_CTLNAME        = npPropInfo (not IPROP_STD_NAME);
  PPROPINFO_STD_INDEX          = npPropInfo (not IPROP_STD_INDEX);
  PPROPINFO_STD_HWND           = npPropInfo (not IPROP_STD_HWND);
  PPROPINFO_STD_BACKCOLOR      = npPropInfo (not IPROP_STD_BACKCOLOR);
  PPROPINFO_STD_FORECOLOR      = npPropInfo (not IPROP_STD_FORECOLOR);
  PPROPINFO_STD_LEFT           = npPropInfo (not IPROP_STD_LEFT);
  PPROPINFO_STD_TOP            = npPropInfo (not IPROP_STD_TOP);
  PPROPINFO_STD_WIDTH          = npPropInfo (not IPROP_STD_WIDTH);
  PPROPINFO_STD_HEIGHT         = npPropInfo (not IPROP_STD_HEIGHT);
  PPROPINFO_STD_ENABLED        = npPropInfo (not IPROP_STD_ENABLED);
  PPROPINFO_STD_VISIBLE        = npPropInfo (not IPROP_STD_VISIBLE);
  PPROPINFO_STD_MOUSEPOINTER   = npPropInfo (not IPROP_STD_MOUSEPOINTER);
  PPROPINFO_STD_CAPTION        = npPropInfo (not IPROP_STD_CAPTION);
  PPROPINFO_STD_FONTNAME       = npPropInfo (not IPROP_STD_FONTNAME);
  PPROPINFO_STD_FONTBOLD       = npPropInfo (not IPROP_STD_FONTBOLD);
  PPROPINFO_STD_FONTITALIC     = npPropInfo (not IPROP_STD_FONTITALIC);
  PPROPINFO_STD_FONTSTRIKE     = npPropInfo (not IPROP_STD_FONTSTRIKE);
  PPROPINFO_STD_FONTUNDER      = npPropInfo (not IPROP_STD_FONTUNDER);
  PPROPINFO_STD_FONTSIZE       = npPropInfo (not IPROP_STD_FONTSIZE);
  PPROPINFO_STD_TABINDEX       = npPropInfo (not IPROP_STD_TABINDEX);
  PPROPINFO_STD_PARENT         = npPropInfo (not IPROP_STD_PARENT);
  PPROPINFO_STD_DRAGMODE       = npPropInfo (not IPROP_STD_DRAGMODE);
  PPROPINFO_STD_DRAGICON       = npPropInfo (not IPROP_STD_DRAGICON);
  PPROPINFO_STD_BORDERSTYLEOFF = npPropInfo (not IPROP_STD_BORDERSTYLEOFF);
  PPROPINFO_STD_TABSTOP        = npPropInfo (not IPROP_STD_TABSTOP);
  PPROPINFO_STD_TAG            = npPropInfo (not IPROP_STD_TAG);
  PPROPINFO_STD_TEXT           = npPropInfo (not IPROP_STD_TEXT);
  PPROPINFO_STD_BORDERSTYLEON  = npPropInfo (not IPROP_STD_BORDERSTYLEON);
  PPROPINFO_STD_CLIPCONTROLS   = npPropInfo (not IPROP_STD_CLIPCONTROLS);
  PPROPINFO_STD_NONE           = npPropInfo (not IPROP_STD_NONE);
  PPROPINFO_STD_HELPCONTEXTID  = npPropInfo (not IPROP_STD_HELPCONTEXTID);
  PPROPINFO_STD_LINKMODE       = npPropInfo (not IPROP_STD_LINKMODE);
  PPROPINFO_STD_LINKITEM       = npPropInfo (not IPROP_STD_LINKITEM);
  PPROPINFO_STD_LINKTOPIC      = npPropInfo (not IPROP_STD_LINKTOPIC);
  PPROPINFO_STD_LINKTIMEOUT    = npPropInfo (not IPROP_STD_LINKTIMEOUT);
  PPROPINFO_STD_LEFTNORUN      = npPropInfo (not IPROP_STD_LEFTNORUN);
  PPROPINFO_STD_TOPNORUN       = npPropInfo (not IPROP_STD_TOPNORUN);
  PPROPINFO_STD_ALIGN          = npPropInfo (not IPROP_STD_ALIGN);
  PPROPINFO_STD_IMEMODE        = npPropInfo (not IPROP_STD_IMEMODE);
  PPROPINFO_STD_DATASOURCE     = npPropInfo (not IPROP_STD_DATASOURCE);
  PPROPINFO_STD_DATAFIELD      = npPropInfo (not IPROP_STD_DATAFIELD);
  PPROPINFO_STD_DATACHANGED    = npPropInfo (not IPROP_STD_DATACHANGED);

  PPROPINFO_STD_LAST           = npPropInfo (not IPROP_STD_LAST);

(*---------------------------------------------------------------------------
// Standard control event list
//---------------------------------------------------------------------------*)
  IEVENT_STD_CLICK      = $0000;
  IEVENT_STD_DBLCLICK   = $0001;
  IEVENT_STD_DRAGDROP   = $0002;
  IEVENT_STD_DRAGOVER   = $0003;
  IEVENT_STD_GOTFOCUS   = $0004;
  IEVENT_STD_KEYDOWN    = $0005;
  IEVENT_STD_KEYPRESS   = $0006;
  IEVENT_STD_KEYUP      = $0007;
  IEVENT_STD_LOSTFOCUS  = $0008;
  IEVENT_STD_MOUSEDOWN  = $0009;
  IEVENT_STD_MOUSEMOVE  = $000A;
  IEVENT_STD_MOUSEUP    = $000B;
  IEVENT_STD_LINKERROR  = $000C;
  IEVENT_STD_LINKOPEN   = $000D;
  IEVENT_STD_LINKCLOSE  = $000E;
  IEVENT_STD_LINKNOTIFY = $000F;
  IEVENT_STD_LINKCHANGE = $0010;
  IEVENT_STD_NONE       = $0FFF;
  IEVENT_STD_LAST       = $0FFF;

  PEVENTINFO_STD_CLICK      = npEventInfo (not IEVENT_STD_CLICK);
  PEVENTINFO_STD_DBLCLICK   = npEventInfo (not IEVENT_STD_DBLCLICK);
  PEVENTINFO_STD_DRAGDROP   = npEventInfo (not IEVENT_STD_DRAGDROP);
  PEVENTINFO_STD_DRAGOVER   = npEventInfo (not IEVENT_STD_DRAGOVER);
  PEVENTINFO_STD_GOTFOCUS   = npEventInfo (not IEVENT_STD_GOTFOCUS);
  PEVENTINFO_STD_KEYDOWN    = npEventInfo (not IEVENT_STD_KEYDOWN);
  PEVENTINFO_STD_KEYPRESS   = npEventInfo (not IEVENT_STD_KEYPRESS);
  PEVENTINFO_STD_KEYUP      = npEventInfo (not IEVENT_STD_KEYUP);
  PEVENTINFO_STD_LOSTFOCUS  = npEventInfo (not IEVENT_STD_LOSTFOCUS);
  PEVENTINFO_STD_MOUSEDOWN  = npEventInfo (not IEVENT_STD_MOUSEDOWN);
  PEVENTINFO_STD_MOUSEMOVE  = npEventInfo (not IEVENT_STD_MOUSEMOVE);
  PEVENTINFO_STD_MOUSEUP    = npEventInfo (not IEVENT_STD_MOUSEUP);
  PEVENTINFO_STD_LINKERROR  = npEventInfo (not IEVENT_STD_LINKERROR);
  PEVENTINFO_STD_LINKOPEN   = npEventInfo (not IEVENT_STD_LINKOPEN);
  PEVENTINFO_STD_LINKCLOSE  = npEventInfo (not IEVENT_STD_LINKCLOSE);
  PEVENTINFO_STD_LINKNOTIFY = npEventInfo (not IEVENT_STD_LINKNOTIFY);
  PEVENTINFO_STD_LINKCHANGE = npEventInfo (not IEVENT_STD_LINKCHANGE);
  PEVENTINFO_STD_NONE       = npEventInfo (not IEVENT_STD_NONE);
  PEVENTINFO_STD_LAST       = npEventInfo (not IEVENT_STD_LAST);

(*---------------------------------------------------------------------------
//---------------------------------------------------------------------------
// Visual Basic messages
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------*)
  VBM__BASE = WM_USER + $0C00;

{ All properties have been loaded.  Must set MODEL_fLoadMsg to receive. }
  VBM_CREATED = VBM__BASE + $00;

{ Form load is complete or dynamic control is fully loaded.  Must set
  MODEL_fLoadMsg to receive. }
  VBM_LOADED = VBM__BASE + $01;

{ Pre-hwnd properties are not yet loaded (no hwnd exists for control yet).
  Must set MODEL_fInitMsg to receive.}
  VBM_INITIALIZE = VBM__BASE + $02;

{ Get a property value. }
{ [wp=iprop, lp=pdata], RetVal=ERR }
  VBM_GETPROPERTY = VBM__BASE + $03;

{ Verify a property value. }
{ [wp=iprop, lp=data], RetVal=ERR }
  VBM_CHECKPROPERTY  = VBM__BASE + $04;

{ Set a property value. }
{ [wp=iprop, lp=data], RetVal=ERR }
  VBM_SETPROPERTY = VBM__BASE + $05;

{ Write a property. }
{ [wp=iprop, lp=pfileref], RetVal=ERR }
  VBM_SAVEPROPERTY = VBM__BASE + $06;

{ Read a property. }
{ [wp=iprop, lp=pfileref], RetVal=ERR }
  VBM_LOADPROPERTY = VBM__BASE + $07;

{ Get the string representation of a property. }
{ [wp=iprop, lp=HSZ FAR *], RetVal=ERR }
  VBM_GETPROPERTYHSZ = VBM__BASE + $08;

{ Create popup window used to change a property. }
{ [wp=iprop, lp=listbox hwnd], RetVal=popup hwnd }
  VBM_INITPROPPOPUP = VBM__BASE + $09;

{ Determine if Paste/PasteLink is allowed. }
{ [wp=PASTETYPE], RetVal=BOOL }
  VBM_QPASTEOK = VBM__BASE + $0A;

  PT_PASTE     = 0;
  PT_PASTELINK = 1;

{ Paste control-specific data from the clipboard. }
{ [wp=PASTETYPE], RetVal=ERR }
  VBM_PASTE = VBM__BASE + $0B;

{ Copy control specific data to the clipboard. }
  VBM_COPY = VBM__BASE + $0C;

{ Control activated upon receipt of a unique mnemonic key. }
  VBM_MNEMONIC = VBM__BASE + $0D;

{ Posted to fire a deferred event. }
{ [wp=event] }
  VBM_FIREEVENT = VBM__BASE + $0E;

{ Another control in drag mode drug over the control. }
{ [lp=pdraginfo] }
  VBM_DRAGOVER = VBM__BASE + $0F;

{ This message is sent to a control that has just had another control }
{ in drag mode dropped on it. }
{ [lp=pdraginfo] }
  VBM_DRAGDROP = VBM__BASE + $10;

{ This message is sent when certain methods are invoked on controls }
{ [wp=METH_, lp=far *alParams], RetVal=ERR }
  VBM_METHOD = VBM__BASE + $11;

  METH_ADDITEM    = $0001;      { args are: cArgs, hszItem, [index]     }
  METH_REMOVEITEM = $0002;      { args are: cArgs, index                }
  METH_REFRESH    = $0003;      { no args (lp == NULL)                  }
  METH_MOVE       = $0004;      { args are: cArgs, left, [top, [width, [height]]] }
  METH_DRAG       = $0005;      { args are: cArgs, [cmd]                }
  METH_LINKSEND   = $0006;      { no args (lp == NULL)                  }
  METH_ZORDER     = $0007;      { args are: cArgs, pos                  }
  METH_CLEAR      = $000A;      { no args (lp == NULL)                  }

{ Clear capture and internal state. }
  VBM_CANCELMODE = VBM__BASE + $12;

{ Paint notification for graphical controls. }
{ [wp=hdc, lp=lprect] }
  VBM_PAINT = VBM__BASE + $14;

{ Hit test for graphical controls. }
{ [lp=lphittest], RetVal = HT_ }
  VBM_HITTEST = VBM__BASE + $15;

  HT_ON           = 7;
  HT_SOLID_NEAR   = 6;
  HT_PATTERN_NEAR = 5;
  HT_HOLLOW_NEAR  = 4;
  HT_SOLID        = 3;
  HT_PATTERN      = 2;
  HT_HOLLOW       = 1;
  HT_MISS         = 0;

type
HITTEST = Record
  pt   : TPoint;
  rect : TRect
end;

lpHitTest = ^HitTest;

const

{ Paint the multi-select handles }
{ [wp=hdc, lp=rect(in hdc coords)] }
  VBM_PAINTMULTISEL = VBM__BASE + $16;

{ Paint the "gray rect" or ctl outline while moving/sizing ctl. }
{ [wp=hdc, lp=rect(in hdc coords)] }
  VBM_PAINTOUTLINE = VBM__BASE + $17;

{ Palette change notification. }
{ [wp=fPalBack(for SelectPalette)], RetVal = return from RealizePalette }
  VBM_PALETTECHANGED = VBM__BASE + $1B;

{ Get control palette. }
{ RetVal = HPALETTE. }
  VBM_GETPALETTE = VBM__BASE + $1C;

{ Enumerate available formats. }
{ [wp=SUPPLIESDATAFORMAT | ACCEPTSDATAFORMAT] }
{ [LOWORD(lp)=ennumeration(0-x)] }
  VBM_LINKENUMFORMATS = VBM__BASE + $1E;

{ Ask a control for it's LINK item name.  Return item name as lpsz in lp. }
{ [wp=LINKSRCASK | LINKSRCTELL] }
{ [lp=lpszItemName] }
  VBM_LINKGETITEMNAME = VBM__BASE + $1F;

{ For Server items and client pokes. }
{ [wp=wFormat lp=LPLINKDATA] }
  VBM_LINKGETDATA = VBM__BASE + $20;

{ Used in client requests, advises, and server pokes. }
{ [wp=wFormat lp=LPLINKDATA] }
  VBM_LINKSETDATA = VBM__BASE + $21;

{ Get help on property or event name. }
{ [LOBYTE(wp)=VBHELP_, HIBYTE(wp)=iprop/ievent, lp=lpmodel] }
  VBM_HELP = VBM__BASE + $22;

  VBHELP_PROP = $0001;
  VBHELP_EVT  = $0002;
  VBHELP_CTL  = $0003;

{ Get default size of control }
{ LOWORD(RetVal)=cx, HIWORD(RetVal)=cy }
  VBM_GETDEFSIZE = VBM__BASE + $23;

{ Just like VBM_SAVE(LOAD)PROPERTY, but saves (loads) as text. }
  VBM_SAVETEXTPROPERTY = VBM__BASE + $24;
  VBM_LOADTEXTPROPERTY = VBM__BASE + $25;

{ Is char a mnemonic for this control? }
{ [wp=char] RetVal=TRUE/FALSE }
  VBM_ISMNEMONIC = VBM__BASE + $26;

{ Does control want to see WM_KEYUP/WM_KEYDOWN for vk?  Sent only }
{ for keys which are normally trapped by VB.  E.g. Tab, Enter, etc. }
{ [wp=vk] RetVal=TRUE/FALSE }
  VBM_WANTSPECIALKEY = VBM__BASE + $27;

{Notifies a control that it has been selected in the property window. }
{[wp=multiselect ON (TRUE) or OFF (FALSE)] }
{[LOWORD(lp)=multiselect count (0 first, n last)] }
  VBM_SELECTED = VBM__BASE + $28;

{ Bound control interface }
  VBM_DATA_INITIATE  = VBM__BASE + $29;
  VBM_DATA_TERMINATE = VBM__BASE + $2A;

  VBM_DATA_AVAILABLE = VBM__BASE + $2B;
  VBM_DATA_GET       = VBM__BASE + $2C;
  VBM_DATA_REQUEST   = VBM__BASE + $2D;
  VBM_DATA_SET       = VBM__BASE + $2E;
  VBM_DATA_METHOD    = VBM__BASE + $2F;


{ Notifications reflected back to control from parent. }
  VBN__BASE = VBM__BASE + $1000;

  VBN_COMMAND      = VBN__BASE + WM_COMMAND;
  VBN_CTLCOLOR     = VBN__BASE + WM_CTLCOLOR;
  VBN_DRAWITEM     = VBN__BASE + WM_DRAWITEM;
  VBN_MEASUREITEM  = VBN__BASE + WM_MEASUREITEM;
  VBN_DELETEITEM   = VBN__BASE + WM_DELETEITEM;
  VBN_VKEYTOITEM   = VBN__BASE + WM_VKEYTOITEM;
  VBN_CHARTOITEM   = VBN__BASE + WM_CHARTOITEM;
  VBN_COMPAREITEM  = VBN__BASE + WM_COMPAREITEM;
  VBN_HSCROLL      = VBN__BASE + WM_HSCROLL;
  VBN_VSCROLL      = VBN__BASE + WM_VSCROLL;
  VBN_PARENTNOTIFY = VBN__BASE + WM_PARENTNOTIFY;

type

fpPropInfo = ^PropInfo;
PropInfo = record
  npszName       : npWord;
  fl             : LongInt;
  OffsetData     : Byte;
  infoData       : Byte;
  dataDefault    : LongInt;
  npszEnumList   : npChar;
  enumMax        : Byte
end;

{---------------------------------------------------------------------------}
{ Control MODEL structure                                                   }
{---------------------------------------------------------------------------}

    lpModel = ^Model;
    Model = Record
		usVersion: Word;                { VB version used by control                    }
		fl: LongInt;                    { Bitfield structure                            }
		pctlproc: pointer;              { the control proc.                             }
		fsClassStyle: Word;             { window class style                            }
		flWndStyle: LongInt;            { default window style                          }
		cbCtlExtra: Word;               { # bytes alloc'd for HCTL structure            }
		idBmpPalette: Word;             { BITMAP id for tool palette                    }
		npszDefCtlName: Word;           { offset of default control name prefix         }
		npszClassName: Word;            { offset of Visual Basic class name             }
		npszParentClassName: Word;      { offset of Parent window class if subclassed   }
		npproplist: Word;               { offset of Property list                       }
		npeventlist: Word;              { offset of Event list                          }
		nDefProp: Byte;                 { index of default property                     }
		nDefEvent: Byte;                { index of default event                        }
		nValueProp : byte;
		usControlVersion : word
	    End;

lpModelInfo = ^ModelInfo;
ModelInfo = record
  usVersion : word;
  lplpModel : ^lpModel
end;

{---------------------------------------------------------------------------}
{ Picture structure                                                         }   
{---------------------------------------------------------------------------}

type
    hPic = THandle;
    lpPic = ^Pic;
    Pic = record
	      picType:  Byte;
	      hData:    THandle;                    { icon, metafile or bitmap handle }
	      xExt, yExt: Integer;                  { metafiles only }
	      picReserved: array [0..3] of Byte;
	  end;


{ General control routines } 

function  VBDerefControl (hctl: HCtl) : pointer; far;
function  VBDefControlProc (hctl: HCtl; hwnd: HWnd; msg: Word; wp: Word; lp: LongInt) : LongInt; far;
function  VBRegisterModel (hmod: THandle; var mdl : Model): Boolean; far;
function  VBGetControlHWnd (gctl: HCtl): HWnd; far;
function  VBGetHInstance: THandle; far;
function  VBGetControlModel (hctl: HCtl): lpModel; far;
function  VBGetControlName (hctl: HCtl; lpszName: PChar): PChar; far;
function  VBGetHWndControl (hwnd: HWnd): HCtl; far;
function  VBSendControlMsg (hctl: HCtl; msg: Word; wp: Word; lp: LongInt): LongInt; far;
function  VBSuperControlProc (hctl: HCtl; msg: Word; wp: Word; lp: LongInt): LongInt; far;
function  VBRecreateControlHWnd (hctl: HCtl): Word; far;
{procedure VBDirtyForm (hctl: HCtl); far;}
function  VBSetErrorMessage (err: PChar; lpstr: PChar): Word; far;
procedure VBGetAppTitle (lpstr: PChar; cbMax: Word); far;
function  VBDialogBoxParam (hInst: THandle; pszTemplateName: PChar; lpDialogFunc: TFarProc; lp: LongInt): Word; far;

{ Management of dynamically allocated strings }

function  VBCreateHsz (seg: THandle; lpszString: PChar): HSz; far;
procedure VBDestroyHsz (hsz: HSz); far;
function  VBDerefHsz (hsz: HSz): PChar; far;
function  VBLockHsz(hsz: HSz): PChar; far;
function  VBUnlockHsz (hsz: HSz): PChar; far;

{ Management of language strings }

function  VBCreateHlstr (pb: Pointer; cbLen: Word): HLStr; far;
procedure VBDestroyHlstr (hlstr: HLStr); far;
function  VBDerefHlstr (hlstr: HLStr): PChar; far;
function  VBGetHlstrLen (hlstr: HLStr): Word; far;
function  VBSetHlstr (hlstr: HLStr; pb: Pointer; cbLen: Word): Word; far;

{ Firing Basic event procedures }

function  VBFireEvent (hctl: HCtl; idEvent: Word; lpParams: Pointer): Word; far;

{ Control property access }

function  VBGetControlProperty (hctl: HCtl; idProp: Word; pdata: Pointer): Word; far;
function  VBSetControlProperty (hctl: HCtl; idProp: Word; data: LongInt): Word; far;

{ Picture management functions }

function  VBAllocPic (lppic: lpPic): HPic; far;
procedure VBFreePic (hpic: HPic); far;
function  VBGetPic (hpic: HPic; lppic: lpPic): HPic; far;
function  VBPicFromCF (var hpic: HPic; hData: THandle; wFormat: Word): Word; far;
function  VBRefPic (hpic: hPic): HPic; far;

{ File IO functions }

function  VBReadFormFile (hformfile: THandle; pb: Pointer; cb: Word): Word; far;
function  VBWriteFormFile (hformfile: THandle; pb: Pointer; cb: Word): Word; far;
function  VBSeekFormFile (hformfile: THandle; offset: LongInt): LongInt; far;
function  VBRelSeekFormFile (hformfile: THandle; offset: LongInt): LongInt; far;
function  VBReadBasicFile (usFileNo: Word; pb: Pointer; cb: Word): Word; far;
function  VBWriteBasicFile (usFileNo: Word; pb: Pointer; cb: Word): Word; far;

{ Conversion functions }

function  VBYPixelsToTwips (Pixels: Integer): LongInt; far;
function  VBXPixelsToTwips (Pixels: Integer): LongInt; far;
function  VBYTwipsToPixels (Twips: LongInt): Integer; far;
function  VBXTwipsToPixels (Twips: LongInt): Integer; far;

{ General routines }

function  VBGetMode: Word; far;


implementation

{$L VBAPI.OBJ}

function  VBDerefControl (hctl: HCtl) : pointer; external;
function  VBDefControlProc (hctl: HCtl; hwnd: HWnd; msg: Word; wp: Word; lp: LongInt): LongInt; external;
function  VBRegisterModel (hmod: THandle; var mdl: Model): Boolean; external;
function  VBGetControlHWnd (gctl: HCtl): HWnd; external;
function  VBGetHInstance: THandle; external;
function  VBGetControlModel (hctl: HCtl): lpModel; external;
function  VBGetControlName (hctl: HCtl; lpszName: PChar): PChar; external;
function  VBGetHWndControl (hwnd: HWnd): HCtl; external;
function  VBSendControlMsg (hctl: HCtl; msg: Word; wp: Word; lp: LongInt): LongInt; external;
function  VBSuperControlProc (hctl: HCtl; msg: Word; wp: Word; lp: LongInt): LongInt; external;
function  VBRecreateControlHWnd (hctl: HCtl): Word; external;
{procedure VBDirtyForm (hctl: HCtl); external;}
function  VBSetErrorMessage (err: PChar; lpstr: PChar): Word; external;
procedure VBGetAppTitle (lpstr: PChar; cbMax: Word); external;
function  VBDialogBoxParam (hInst: THandle; pszTemplateName: PChar; lpDialogFunc: TFarProc; lp: LongInt): Word; external;
function  VBCreateHsz (seg: THandle; lpszString: PChar): HSz; external;
procedure VBDestroyHsz (hsz: HSz); external;
function  VBDerefHsz (hsz: HSz): PChar; external;
function  VBLockHsz(hsz: HSz): PChar; external;
function  VBUnlockHsz (hsz: HSz): PChar; external;
function  VBCreateHlstr (pb: Pointer; cbLen: Word): HLStr; external;
procedure VBDestroyHlstr (hlstr: HLStr); external;
function  VBDerefHlstr (hlstr: HLStr): PChar; external;
function  VBGetHlstrLen (hlstr: HLStr): Word; external;
function  VBSetHlstr (hlstr: HLStr; pb: Pointer; cbLen: Word): Word; external;
function  VBFireEvent (hctl: HCtl; idEvent: Word; lpParams: Pointer): Word; external;
function  VBGetControlProperty (hctl: HCtl; idProp: Word; pdata: Pointer): Word; external;
function  VBSetControlProperty (hctl: HCtl; idProp: Word; data: LongInt): Word; external;
function  VBAllocPic (lppic: lpPic): HPic; external;
procedure VBFreePic (hpic: HPic); external;
function  VBGetPic (hpic: HPic; lppic: lpPic): HPic; external;
function  VBPicFromCF (var hpic: HPic; hData: THandle; wFormat: Word): Word; external;
function  VBRefPic (hpic: hPic): HPic; external;
function  VBReadFormFile (hformfile: THandle; pb: Pointer; cb: Word): Word; external;
function  VBWriteFormFile (hformfile: THandle; pb: Pointer; cb: Word): Word; external;
function  VBSeekFormFile (hformfile: THandle; offset: LongInt): LongInt; external;
function  VBRelSeekFormFile (hformfile: THandle; offset: LongInt): LongInt; external;
function  VBReadBasicFile (usFileNo: Word; pb: Pointer; cb: Word): Word; external;
function  VBWriteBasicFile (usFileNo: Word; pb: Pointer; cb: Word): Word; external;
function  VBYPixelsToTwips (Pixels: Integer): LongInt; external;
function  VBXPixelsToTwips (Pixels: Integer): LongInt; external;
function  VBYTwipsToPixels (Twips: LongInt): Integer; external;
function  VBXTwipsToPixels (Twips: LongInt): Integer; external;
function  VBGetMode: Word; external;

end.

