{A PCX decoding demo program.}
{Copyright 1993, Mark D. Rafn, MDRUtils(tm)}
{Target: Windows 3.1}
{$R PCX}
program Winpcx;

uses
	WinProcs, WinTypes, OWindows, ODialogs,
	Strings, BWCC, OStdDlgs, uCanvas, uPcxWin;

const
	AppName: PChar = 'WinPcx';
	cm_FileOpen = 101;
  cm_HelpAbout = 102;
  cm_ZoomIn = 103;
  cm_ZoomOut = 104;
  cm_ActualSize = 105;
  wm_About = wm_User;
  ZoomScale = 2;

type
{--------------- Main Window Object ---------------}
PWinpcx = ^TWinpcx;
TWinpcx = object(TApplication)
	procedure InitMainWindow; virtual;
end;

{--------------- Main Window of the application -------------------}
PMainWindow = ^TMainWindow;
TMainWindow = object(TWindow)
	State: TState;
  Canvas: PCanvas;
  {Canvas is owned by the parent window so it is automatically disposed of}
	constructor Init(AParent: PWindowsObject; ATitle: PChar);
	procedure GetWindowClass(var AWndClass: TWndClass); virtual;
	function  GetClassName: PChar; virtual;
  procedure SetupWindow; virtual;
  procedure EnableZoomMenu;
  procedure DisableZoomMenu;

  procedure CMHelpAbout(var Msg: TMessage);
  	virtual cm_First + cm_HelpAbout;
  procedure CMZoomIn(var Msg: TMessage);
  	virtual cm_First + cm_ZoomIn;
  procedure CMZoomOut(var Msg: TMessage);
  	virtual cm_First + cm_ZoomOut;
  procedure CMAcutalSize(var Msg: TMessage);
  	virtual cm_First + cm_ActualSize;
	procedure CMOpenFile(var Msg: TMessage);
  	virtual cm_First + cm_FileOpen;

  procedure WMAbout(var Msg: TMessage);
  	virtual wm_First + wm_About;
  procedure WMSize(var Msg: TMessage);
    virtual wm_First + wm_Size;
end;

constructor TMainWindow.Init(AParent: PWindowsObject; ATitle: PChar);
var
	Width, Height: integer;
begin
	inherited Init(AParent, ATitle);
  Width := GetSystemMetrics(sm_cXScreen);
  Height := GetSystemMetrics(sm_cYScreen);
  with Attr do
  begin
	  Menu := LoadMenu(HInstance, 'COMMANDS');
    W := 640;
    H := 480;
    X := (Width div 2) - (W div 2);
    Y := (Height div 2) - (H div 2);
    if X < 0 then X := 0;
    if Y < 0 then Y := 0;
  end;
  Canvas := nil;
end;

procedure TMainWindow.SetUpWindow;
var
	R: TRect;
begin
	inherited SetupWindow;
	GetClientRect(HWindow, R);
  with State, R do
  begin
  	BitmapSize.x := Right - Left;
    BitmapSize.y := Bottom - Top;
    Zoom := false;
  end;
  DisableZoomMenu;
end;

function TMainWindow.GetClassName : PChar;
begin GetClassName := AppName; end;

procedure TMainWindow.GetWindowClass(var AWndClass: TWndClass);
begin
	TWindow.GetWindowClass(AWndClass);
  AWndClass.hbrBackground := CreateSolidBrush($828282);
  AWndClass.hIcon := LoadIcon(HInstance, 'MDRIcon');
end;

procedure TMainWindow.EnableZoomMenu;
begin
	EnableMenuItem(Attr.Menu, cm_ZoomIn, mf_Enabled);
	EnableMenuItem(Attr.Menu, cm_ZoomOut, mf_Enabled);
	EnableMenuItem(Attr.Menu, cm_ActualSize, mf_Enabled);
end;

procedure TMainWindow.DisableZoomMenu;
begin
	EnableMenuItem(Attr.Menu, cm_ZoomIn, mf_Grayed);
	EnableMenuItem(Attr.Menu, cm_ZoomOut, mf_Grayed);
	EnableMenuItem(Attr.Menu, cm_ActualSize, mf_Grayed);
end;

procedure TMainWindow.CMHelpAbout(var Msg: TMessage);
begin
	Application^.ExecDialog(New(PDialog, Init(@self, 'MDRABOUT')));
end;

procedure TMainWindow.CMZoomIn(var Msg: TMessage);
begin
	with State do
  begin
  	BitmapSize.x := BitmapSize.x * ZoomScale;
    BitmapSize.y := Bitmapsize.y * ZoomScale;
  end;
	State.Zoom := true;
  WMSize(Msg);
end;

procedure TMainWindow.CMZoomOut(var Msg: TMessage);
begin
	with State do
  begin
  	BitmapSize.x := BitmapSize.x div ZoomScale;
    BitmapSize.y := Bitmapsize.y div ZoomScale;
  end;
	State.Zoom := true;
  WMSize(Msg);
end;

procedure TMainWindow.CMAcutalSize(var Msg: TMessage);
begin
	State.BitmapSize := State.OriginalSize;
  State.Zoom := false;
  WMSize(Msg);
end;

procedure TMainWindow.WMAbout(var Msg: TMessage);
begin CMHelpAbout(Msg); end;

procedure TMainwindow.WMSize(var Msg: TMessage);
var
	R: TRect;
begin
	TWindow.WMSize(Msg);
	GetClientRect(HWindow, R);
  if Assigned(Canvas) then
	  with R do
  		Canvas^.MoveSelf(Left + 10, Top + 10, Right - 10, Bottom - 10, true);
end;

procedure TMainWindow.CMOpenFile(var Msg: TMessage);
var
  Filename: array[0..80] of char;
  R: TRect;
  Cursor: hCursor;
begin
	StrCopy(Filename,'*.pcx');
	if Application^.ExecDialog(New(PFileDialog,
		Init(@self,MakeIntResource(sd_FileOpen), Filename))) = id_ok then
	begin
  	Cursor := SetCursor(LoadCursor(0,idc_Wait));
    DisableZoomMenu;
    if Assigned(Canvas) then Canvas^.CloseWindow;
  	Canvas := PCanvas(Application^.MakeWindow(New(PCanvas, Init(@Self, @State))));
    if Canvas^.LoadFile(Filename) <> 1 then
    begin
			PCXErrorMessage(HWindow);
      Canvas^.CloseWindow;
    end
    else
    begin
      GetClientRect(HWindow, R);
      Canvas^.MoveSelf(R.Left + 10, R.Top + 10, R.Right - 10, R.Bottom -10, true);
      EnableZoomMenu;
    end;
    SetCursor(Cursor);
	end;
end;

{ Create the application's main window }
procedure TWinpcx.InitMainWindow;
begin
	MainWindow := New(PMainWindow, Init(nil, AppName));
end;

var
	MainApp: TWinpcx;

begin
	MainApp.Init(AppName);
  PostMessage(MainApp.MainWindow^.HWindow, wm_About, 0, 0);
  MainApp.Run;
  MainApp.Done;
end.
